@@ -128,12 +128,11 @@ class TestVersioning(TestCase, AssertsCompiledSQL):
     def test_w_mapper_versioning(self):
         class SomeClass(Versioned, self.Base, ComparableEntity):
             __tablename__ = "sometable"
+            use_mapper_versioning = True
 
             id = Column(Integer, primary_key=True)
             name = Column(String(50))
 
-        SomeClass.__mapper__.version_id_col = SomeClass.__table__.c.version
-
         self.create_tables()
         sess = self.session
         sc = SomeClass(name="sc1")
@@ -461,10 +460,10 @@ class TestVersioning(TestCase, AssertsCompiledSQL):
         sess.commit()
 
         actual_changed_base = sess.scalar(
-            select([BaseClass.__history_mapper__.local_table.c.changed])
+            select(BaseClass.__history_mapper__.local_table.c.changed)
         )
         actual_changed_sub = sess.scalar(
-            select([SubClass.__history_mapper__.local_table.c.changed])
+            select(SubClass.__history_mapper__.local_table.c.changed)
         )
         h1 = sess.query(BaseClassHistory).first()
         eq_(h1.changed, actual_changed_base)
