@@ -1,18 +1,18 @@
 # testing/mock.py
-# Copyright (C) 2005-2020 the SQLAlchemy authors and contributors
+# Copyright (C) 2005-2022 the SQLAlchemy authors and contributors
 # <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
-# the MIT License: http://www.opensource.org/licenses/mit-license.php
+# the MIT License: https://www.opensource.org/licenses/mit-license.php
 
 """Import stub for mock library.
 """
 from __future__ import absolute_import
 
-from ..util import py33
+from ..util import py3k
 
 
-if py33:
+if py3k:
     from unittest.mock import MagicMock
     from unittest.mock import Mock
     from unittest.mock import call
