@@ -4,6 +4,7 @@
 import datetime
 import decimal
 import os
+import random
 
 from sqlalchemy import bindparam
 from sqlalchemy import cast
@@ -51,6 +52,10 @@ from sqlalchemy.util import py2k
 from sqlalchemy.util import u
 
 
+def exec_sql(conn, sql, *args, **kwargs):
+    return conn.exec_driver_sql(sql, *args, **kwargs)
+
+
 class DialectTypesTest(fixtures.TestBase, AssertsCompiledSQL):
     __dialect__ = oracle.OracleDialect()
 
@@ -178,16 +183,19 @@ class DialectTypesTest(fixtures.TestBase, AssertsCompiledSQL):
     def test_interval(self, type_, expected):
         self.assert_compile(type_, expected)
 
+    def test_interval_coercion_literal(self):
+        expr = column("bar", oracle.INTERVAL) == datetime.timedelta(days=1)
+        eq_(expr.right.type._type_affinity, sqltypes.Interval)
+
 
 class TypesTest(fixtures.TestBase):
     __only_on__ = "oracle"
     __dialect__ = oracle.OracleDialect()
     __backend__ = True
 
-    @testing.combinations((CHAR,), (NCHAR,))
-    @testing.provide_metadata
-    def test_fixed_char(self, char_type):
-        m = self.metadata
+    @testing.combinations((CHAR,), (NCHAR,), argnames="char_type")
+    def test_fixed_char(self, metadata, connection, char_type):
+        m = metadata
         t = Table(
             "t1",
             m,
@@ -200,85 +208,79 @@ class TypesTest(fixtures.TestBase):
         else:
             v1, v2, v3 = "value 1", "value 2", "value 3"
 
-        with testing.db.begin() as conn:
-            t.create(conn)
-            conn.execute(
-                t.insert(),
+        t.create(connection)
+        connection.execute(
+            t.insert(),
+            [
                 dict(id=1, data=v1),
                 dict(id=2, data=v2),
                 dict(id=3, data=v3),
-            )
+            ],
+        )
 
-            eq_(
-                conn.execute(t.select().where(t.c.data == v2)).fetchall(),
-                [(2, "value 2                       ")],
-            )
+        eq_(
+            connection.execute(t.select().where(t.c.data == v2)).fetchall(),
+            [(2, "value 2                       ")],
+        )
 
-            m2 = MetaData()
-            t2 = Table("t1", m2, autoload_with=conn)
-            is_(type(t2.c.data.type), char_type)
-            eq_(
-                conn.execute(t2.select().where(t2.c.data == v2)).fetchall(),
-                [(2, "value 2                       ")],
-            )
+        m2 = MetaData()
+        t2 = Table("t1", m2, autoload_with=connection)
+        is_(type(t2.c.data.type), char_type)
+        eq_(
+            connection.execute(t2.select().where(t2.c.data == v2)).fetchall(),
+            [(2, "value 2                       ")],
+        )
 
     @testing.requires.returning
-    @testing.provide_metadata
-    def test_int_not_float(self):
-        m = self.metadata
+    def test_int_not_float(self, metadata, connection):
+        m = metadata
         t1 = Table("t1", m, Column("foo", Integer))
-        t1.create()
-        r = t1.insert().values(foo=5).returning(t1.c.foo).execute()
+        t1.create(connection)
+        r = connection.execute(t1.insert().values(foo=5).returning(t1.c.foo))
         x = r.scalar()
         assert x == 5
         assert isinstance(x, int)
 
-        x = t1.select().scalar()
+        x = connection.scalar(t1.select())
         assert x == 5
         assert isinstance(x, int)
 
     @testing.requires.returning
-    @testing.provide_metadata
-    def test_int_not_float_no_coerce_decimal(self):
+    def test_int_not_float_no_coerce_decimal(self, metadata):
         engine = testing_engine(options=dict(coerce_to_decimal=False))
 
-        m = self.metadata
+        m = metadata
         t1 = Table("t1", m, Column("foo", Integer))
-        t1.create()
-        r = engine.execute(t1.insert().values(foo=5).returning(t1.c.foo))
-        x = r.scalar()
-        assert x == 5
-        assert isinstance(x, int)
+        with engine.begin() as conn:
+            t1.create(conn)
+            r = conn.execute(t1.insert().values(foo=5).returning(t1.c.foo))
+            x = r.scalar()
+            assert x == 5
+            assert isinstance(x, int)
 
-        x = t1.select().scalar()
-        assert x == 5
-        assert isinstance(x, int)
+            x = conn.execute(t1.select()).scalar()
+            assert x == 5
+            assert isinstance(x, int)
 
-    @testing.provide_metadata
-    def test_rowid(self):
-        metadata = self.metadata
+    def test_rowid(self, metadata, connection):
         t = Table("t1", metadata, Column("x", Integer))
-        t.create()
-        t.insert().execute(x=5)
-        s1 = select([t])
-        s2 = select([column("rowid")]).select_from(s1)
-        rowid = s2.scalar()
+
+        t.create(connection)
+        connection.execute(t.insert(), {"x": 5})
+        s1 = select(t).subquery()
+        s2 = select(column("rowid")).select_from(s1)
+        rowid = connection.scalar(s2)
 
         # the ROWID type is not really needed here,
         # as cx_oracle just treats it as a string,
         # but we want to make sure the ROWID works...
         rowid_col = column("rowid", oracle.ROWID)
-        s3 = select([t.c.x, rowid_col]).where(
+        s3 = select(t.c.x, rowid_col).where(
             rowid_col == cast(rowid, oracle.ROWID)
         )
-        eq_(s3.select().execute().fetchall(), [(5, rowid)])
+        eq_(connection.execute(s3).fetchall(), [(5, rowid)])
 
-    @testing.fails_on(
-        "+zxjdbc", "Not yet known how to pass values of the " "INTERVAL type"
-    )
-    @testing.provide_metadata
-    def test_interval(self):
-        metadata = self.metadata
+    def test_interval(self, metadata, connection):
         interval_table = Table(
             "intervaltable",
             metadata,
@@ -287,16 +289,19 @@ class TypesTest(fixtures.TestBase):
             ),
             Column("day_interval", oracle.INTERVAL(day_precision=3)),
         )
-        metadata.create_all()
-        interval_table.insert().execute(
-            day_interval=datetime.timedelta(days=35, seconds=5743)
+        metadata.create_all(connection)
+        connection.execute(
+            interval_table.insert(),
+            dict(day_interval=datetime.timedelta(days=35, seconds=5743)),
+        )
+        row = connection.execute(interval_table.select()).first()
+        eq_(
+            row._mapping["day_interval"],
+            datetime.timedelta(days=35, seconds=5743),
         )
-        row = interval_table.select().execute().first()
-        eq_(row["day_interval"], datetime.timedelta(days=35, seconds=5743))
 
-    @testing.provide_metadata
-    def test_numerics(self):
-        m = self.metadata
+    def test_numerics(self, metadata, connection):
+        m = metadata
         t1 = Table(
             "t1",
             m,
@@ -309,24 +314,27 @@ class TypesTest(fixtures.TestBase):
             Column("numbercol2", oracle.NUMBER(9, 3)),
             Column("numbercol3", oracle.NUMBER),
         )
-        t1.create()
-        t1.insert().execute(
-            intcol=1,
-            numericcol=5.2,
-            floatcol1=6.5,
-            floatcol2=8.5,
-            doubleprec=9.5,
-            numbercol1=12,
-            numbercol2=14.85,
-            numbercol3=15.76,
+        t1.create(connection)
+        connection.execute(
+            t1.insert(),
+            dict(
+                intcol=1,
+                numericcol=5.2,
+                floatcol1=6.5,
+                floatcol2=8.5,
+                doubleprec=9.5,
+                numbercol1=12,
+                numbercol2=14.85,
+                numbercol3=15.76,
+            ),
         )
 
-        m2 = MetaData(testing.db)
-        t2 = Table("t1", m2, autoload=True)
+        m2 = MetaData()
+        t2 = Table("t1", m2, autoload_with=connection)
 
         for row in (
-            t1.select().execute().first(),
-            t2.select().execute().first(),
+            connection.execute(t1.select()).first(),
+            connection.execute(t2.select()).first(),
         ):
             for i, (val, type_) in enumerate(
                 (
@@ -346,93 +354,91 @@ class TypesTest(fixtures.TestBase):
                     type_,
                 )
 
-    @testing.provide_metadata
-    def test_numeric_infinity_float(self):
-        m = self.metadata
+    def test_numeric_infinity_float(self, metadata, connection):
+        m = metadata
         t1 = Table(
             "t1",
             m,
             Column("intcol", Integer),
             Column("numericcol", oracle.BINARY_DOUBLE(asdecimal=False)),
         )
-        t1.create()
-        t1.insert().execute(
+        t1.create(connection)
+        connection.execute(
+            t1.insert(),
             [
                 dict(intcol=1, numericcol=float("inf")),
                 dict(intcol=2, numericcol=float("-inf")),
-            ]
+            ],
         )
 
         eq_(
-            select([t1.c.numericcol])
-            .order_by(t1.c.intcol)
-            .execute()
-            .fetchall(),
-            [(float("inf"),), (float("-inf"),)],
-        )
-
-        eq_(
-            testing.db.execute(
-                "select numericcol from t1 order by intcol"
+            connection.execute(
+                select(t1.c.numericcol).order_by(t1.c.intcol)
             ).fetchall(),
             [(float("inf"),), (float("-inf"),)],
         )
 
-    @testing.provide_metadata
-    def test_numeric_infinity_decimal(self):
-        m = self.metadata
+        eq_(
+            exec_sql(
+                connection, "select numericcol from t1 order by intcol"
+            ).fetchall(),
+            [(float("inf"),), (float("-inf"),)],
+        )
+
+    def test_numeric_infinity_decimal(self, metadata, connection):
+        m = metadata
         t1 = Table(
             "t1",
             m,
             Column("intcol", Integer),
             Column("numericcol", oracle.BINARY_DOUBLE(asdecimal=True)),
         )
-        t1.create()
-        t1.insert().execute(
+        t1.create(connection)
+        connection.execute(
+            t1.insert(),
             [
                 dict(intcol=1, numericcol=decimal.Decimal("Infinity")),
                 dict(intcol=2, numericcol=decimal.Decimal("-Infinity")),
-            ]
+            ],
         )
 
         eq_(
-            select([t1.c.numericcol])
-            .order_by(t1.c.intcol)
-            .execute()
-            .fetchall(),
-            [(decimal.Decimal("Infinity"),), (decimal.Decimal("-Infinity"),)],
-        )
-
-        eq_(
-            testing.db.execute(
-                "select numericcol from t1 order by intcol"
+            connection.execute(
+                select(t1.c.numericcol).order_by(t1.c.intcol)
             ).fetchall(),
             [(decimal.Decimal("Infinity"),), (decimal.Decimal("-Infinity"),)],
         )
 
-    @testing.provide_metadata
-    def test_numeric_nan_float(self):
-        m = self.metadata
+        eq_(
+            exec_sql(
+                connection, "select numericcol from t1 order by intcol"
+            ).fetchall(),
+            [(decimal.Decimal("Infinity"),), (decimal.Decimal("-Infinity"),)],
+        )
+
+    def test_numeric_nan_float(self, metadata, connection):
+        m = metadata
         t1 = Table(
             "t1",
             m,
             Column("intcol", Integer),
             Column("numericcol", oracle.BINARY_DOUBLE(asdecimal=False)),
         )
-        t1.create()
-        t1.insert().execute(
+        t1.create(connection)
+        connection.execute(
+            t1.insert(),
             [
                 dict(intcol=1, numericcol=float("nan")),
                 dict(intcol=2, numericcol=float("-nan")),
-            ]
+            ],
         )
 
         eq_(
             [
                 tuple(str(col) for col in row)
-                for row in select([t1.c.numericcol])
-                .order_by(t1.c.intcol)
-                .execute()
+                for row in connection.execute(
+                    select(t1.c.numericcol).order_by(t1.c.intcol)
+                )
             ],
             [("nan",), ("nan",)],
         )
@@ -440,8 +446,8 @@ class TypesTest(fixtures.TestBase):
         eq_(
             [
                 tuple(str(col) for col in row)
-                for row in testing.db.execute(
-                    "select numericcol from t1 order by intcol"
+                for row in exec_sql(
+                    connection, "select numericcol from t1 order by intcol"
                 )
             ],
             [("nan",), ("nan",)],
@@ -449,9 +455,8 @@ class TypesTest(fixtures.TestBase):
 
     # needs https://github.com/oracle/python-cx_Oracle/
     # issues/184#issuecomment-391399292
-    @testing.provide_metadata
-    def _dont_test_numeric_nan_decimal(self):
-        m = self.metadata
+    def _dont_test_numeric_nan_decimal(self, metadata, connection):
+        m = metadata
         t1 = Table(
             "t1",
             m,
@@ -467,30 +472,24 @@ class TypesTest(fixtures.TestBase):
         )
 
         eq_(
-            select([t1.c.numericcol])
-            .order_by(t1.c.intcol)
-            .execute()
-            .fetchall(),
+            select(t1.c.numericcol).order_by(t1.c.intcol).execute().fetchall(),
             [(decimal.Decimal("NaN"),), (decimal.Decimal("NaN"),)],
         )
 
         eq_(
-            testing.db.execute(
-                "select numericcol from t1 order by intcol"
+            exec_sql(
+                connection, "select numericcol from t1 order by intcol"
             ).fetchall(),
             [(decimal.Decimal("NaN"),), (decimal.Decimal("NaN"),)],
         )
 
-    @testing.provide_metadata
-    def test_numerics_broken_inspection(self):
+    def test_numerics_broken_inspection(self, metadata, connection):
         """Numeric scenarios where Oracle type info is 'broken',
         returning us precision, scale of the form (0, 0) or (0, -127).
         We convert to Decimal and let int()/float() processors take over.
 
         """
 
-        metadata = self.metadata
-
         # this test requires cx_oracle 5
 
         foo = Table(
@@ -502,21 +501,22 @@ class TypesTest(fixtures.TestBase):
             Column("nidata", Numeric(5, 0)),
             Column("fdata", Float()),
         )
-        foo.create()
+        foo.create(connection)
 
-        foo.insert().execute(
+        connection.execute(
+            foo.insert(),
             {
                 "idata": 5,
                 "ndata": decimal.Decimal("45.6"),
                 "ndata2": decimal.Decimal("45.0"),
                 "nidata": decimal.Decimal("53"),
                 "fdata": 45.68392,
-            }
+            },
         )
 
         stmt = "SELECT idata, ndata, ndata2, nidata, fdata FROM foo"
 
-        row = testing.db.execute(stmt).fetchall()[0]
+        row = exec_sql(connection, stmt).fetchall()[0]
         eq_(
             [type(x) for x in row],
             [int, decimal.Decimal, decimal.Decimal, int, float],
@@ -552,7 +552,7 @@ class TypesTest(fixtures.TestBase):
             (SELECT CAST((SELECT fdata FROM foo) AS FLOAT) FROM DUAL) AS fdata
         FROM dual
         """
-        row = testing.db.execute(stmt).fetchall()[0]
+        row = exec_sql(connection, stmt).fetchall()[0]
         eq_(
             [type(x) for x in row],
             [int, decimal.Decimal, int, int, decimal.Decimal],
@@ -562,7 +562,7 @@ class TypesTest(fixtures.TestBase):
             (5, decimal.Decimal("45.6"), 45, 53, decimal.Decimal("45.68392")),
         )
 
-        row = testing.db.execute(
+        row = connection.execute(
             text(stmt).columns(
                 idata=Integer(),
                 ndata=Numeric(20, 2),
@@ -609,7 +609,7 @@ class TypesTest(fixtures.TestBase):
         )
         WHERE ROWNUM >= 0) anon_1
         """
-        row = testing.db.execute(stmt).fetchall()[0]
+        row = exec_sql(connection, stmt).fetchall()[0]
         eq_(
             [type(x) for x in row],
             [int, decimal.Decimal, int, int, decimal.Decimal],
@@ -619,7 +619,7 @@ class TypesTest(fixtures.TestBase):
             (5, decimal.Decimal("45.6"), 45, 53, decimal.Decimal("45.68392")),
         )
 
-        row = testing.db.execute(
+        row = connection.execute(
             text(stmt).columns(
                 anon_1_idata=Integer(),
                 anon_1_ndata=Numeric(20, 2),
@@ -643,7 +643,7 @@ class TypesTest(fixtures.TestBase):
             ),
         )
 
-        row = testing.db.execute(
+        row = connection.execute(
             text(stmt).columns(
                 anon_1_idata=Integer(),
                 anon_1_ndata=Numeric(20, 2, asdecimal=False),
@@ -659,22 +659,23 @@ class TypesTest(fixtures.TestBase):
 
     def test_numeric_no_coerce_decimal_mode(self):
         engine = testing_engine(options=dict(coerce_to_decimal=False))
+        with engine.connect() as conn:
+            # raw SQL no longer coerces to decimal
+            value = exec_sql(conn, "SELECT 5.66 FROM DUAL").scalar()
+            assert isinstance(value, float)
 
-        # raw SQL no longer coerces to decimal
-        value = engine.scalar("SELECT 5.66 FROM DUAL")
-        assert isinstance(value, float)
-
-        # explicit typing still *does* coerce to decimal
-        # (change in 1.2)
-        value = engine.scalar(
-            text("SELECT 5.66 AS foo FROM DUAL").columns(
-                foo=Numeric(4, 2, asdecimal=True)
+            # explicit typing still *does* coerce to decimal
+            # (change in 1.2)
+            value = conn.scalar(
+                text("SELECT 5.66 AS foo FROM DUAL").columns(
+                    foo=Numeric(4, 2, asdecimal=True)
+                )
             )
-        )
-        assert isinstance(value, decimal.Decimal)
+            assert isinstance(value, decimal.Decimal)
 
+    def test_numeric_coerce_decimal_mode(self, connection):
         # default behavior is raw SQL coerces to decimal
-        value = testing.db.scalar("SELECT 5.66 FROM DUAL")
+        value = exec_sql(connection, "SELECT 5.66 FROM DUAL").scalar()
         assert isinstance(value, decimal.Decimal)
 
     @testing.combinations(
@@ -714,7 +715,7 @@ class TypesTest(fixtures.TestBase):
             cx_oracle_result = cursor.fetchone()[0]
             cursor.close()
 
-            sqla_result = conn.scalar(stmt)
+            sqla_result = conn.exec_driver_sql(stmt).scalar()
 
             eq_(sqla_result, cx_oracle_result)
 
@@ -722,17 +723,18 @@ class TypesTest(fixtures.TestBase):
     @testing.fails_if(
         testing.requires.python3, "cx_oracle always returns unicode on py3k"
     )
-    def test_coerce_to_unicode(self):
+    def test_coerce_to_unicode(self, connection):
         engine = testing_engine(options=dict(coerce_to_unicode=False))
-        value = engine.scalar("SELECT 'hello' FROM DUAL")
-        assert isinstance(value, util.binary_type)
+        with engine.connect() as conn_no_coerce:
+            value = exec_sql(
+                conn_no_coerce, "SELECT 'hello' FROM DUAL"
+            ).scalar()
+            assert isinstance(value, util.binary_type)
 
-        value = testing.db.scalar("SELECT 'hello' FROM DUAL")
+        value = exec_sql(connection, "SELECT 'hello' FROM DUAL").scalar()
         assert isinstance(value, util.text_type)
 
-    @testing.provide_metadata
-    def test_reflect_dates(self):
-        metadata = self.metadata
+    def test_reflect_dates(self, metadata, connection):
         Table(
             "date_types",
             metadata,
@@ -742,9 +744,9 @@ class TypesTest(fixtures.TestBase):
             Column("d4", TIMESTAMP(timezone=True)),
             Column("d5", oracle.INTERVAL(second_precision=5)),
         )
-        metadata.create_all()
-        m = MetaData(testing.db)
-        t1 = Table("date_types", m, autoload=True)
+        metadata.create_all(connection)
+        m = MetaData()
+        t1 = Table("date_types", m, autoload_with=connection)
         assert isinstance(t1.c.d1.type, oracle.DATE)
         assert isinstance(t1.c.d1.type, DateTime)
         assert isinstance(t1.c.d2.type, oracle.DATE)
@@ -758,87 +760,78 @@ class TypesTest(fixtures.TestBase):
     def _dont_test_reflect_all_types_schema(self):
         types_table = Table(
             "all_types",
-            MetaData(testing.db),
+            MetaData(),
             Column("owner", String(30), primary_key=True),
             Column("type_name", String(30), primary_key=True),
-            autoload=True,
+            autoload_with=testing.db,
             oracle_resolve_synonyms=True,
         )
         for row in types_table.select().execute().fetchall():
             [row[k] for k in row.keys()]
 
-    @testing.provide_metadata
-    def test_raw_roundtrip(self):
-        metadata = self.metadata
+    def test_raw_roundtrip(self, metadata, connection):
         raw_table = Table(
             "raw",
             metadata,
             Column("id", Integer, primary_key=True),
             Column("data", oracle.RAW(35)),
         )
-        metadata.create_all()
-        testing.db.execute(raw_table.insert(), id=1, data=b("ABCDEF"))
-        eq_(testing.db.execute(raw_table.select()).first(), (1, b("ABCDEF")))
+        metadata.create_all(connection)
+        connection.execute(raw_table.insert(), dict(id=1, data=b("ABCDEF")))
+        eq_(connection.execute(raw_table.select()).first(), (1, b("ABCDEF")))
 
-    @testing.provide_metadata
-    def test_reflect_nvarchar(self):
-        metadata = self.metadata
+    def test_reflect_nvarchar(self, metadata, connection):
         Table(
             "tnv",
             metadata,
             Column("nv_data", sqltypes.NVARCHAR(255)),
             Column("c_data", sqltypes.NCHAR(20)),
         )
-        metadata.create_all()
-        m2 = MetaData(testing.db)
-        t2 = Table("tnv", m2, autoload=True)
+        metadata.create_all(connection)
+        m2 = MetaData()
+        t2 = Table("tnv", m2, autoload_with=connection)
         assert isinstance(t2.c.nv_data.type, sqltypes.NVARCHAR)
         assert isinstance(t2.c.c_data.type, sqltypes.NCHAR)
 
         if testing.against("oracle+cx_oracle"):
             assert isinstance(
-                t2.c.nv_data.type.dialect_impl(testing.db.dialect),
+                t2.c.nv_data.type.dialect_impl(connection.dialect),
                 cx_oracle._OracleUnicodeStringNCHAR,
             )
 
             assert isinstance(
-                t2.c.c_data.type.dialect_impl(testing.db.dialect),
+                t2.c.c_data.type.dialect_impl(connection.dialect),
                 cx_oracle._OracleNChar,
             )
 
         data = u("m’a réveillé.")
-        with testing.db.connect() as conn:
-            conn.execute(t2.insert(), dict(nv_data=data, c_data=data))
-            nv_data, c_data = conn.execute(t2.select()).first()
-            eq_(nv_data, data)
-            eq_(c_data, data + (" " * 7))  # char is space padded
-            assert isinstance(nv_data, util.text_type)
-            assert isinstance(c_data, util.text_type)
+        connection.execute(t2.insert(), dict(nv_data=data, c_data=data))
+        nv_data, c_data = connection.execute(t2.select()).first()
+        eq_(nv_data, data)
+        eq_(c_data, data + (" " * 7))  # char is space padded
+        assert isinstance(nv_data, util.text_type)
+        assert isinstance(c_data, util.text_type)
 
-    @testing.provide_metadata
-    def test_reflect_unicode_no_nvarchar(self):
-        metadata = self.metadata
+    def test_reflect_unicode_no_nvarchar(self, metadata, connection):
         Table("tnv", metadata, Column("data", sqltypes.Unicode(255)))
-        metadata.create_all()
-        m2 = MetaData(testing.db)
-        t2 = Table("tnv", m2, autoload=True)
+        metadata.create_all(connection)
+        m2 = MetaData()
+        t2 = Table("tnv", m2, autoload_with=connection)
         assert isinstance(t2.c.data.type, sqltypes.VARCHAR)
 
         if testing.against("oracle+cx_oracle"):
             assert isinstance(
-                t2.c.data.type.dialect_impl(testing.db.dialect),
+                t2.c.data.type.dialect_impl(connection.dialect),
                 cx_oracle._OracleString,
             )
 
         data = u("m’a réveillé.")
-        t2.insert().execute(data=data)
-        res = t2.select().execute().first()["data"]
+        connection.execute(t2.insert(), {"data": data})
+        res = connection.execute(t2.select()).first().data
         eq_(res, data)
         assert isinstance(res, util.text_type)
 
-    @testing.provide_metadata
-    def test_char_length(self):
-        metadata = self.metadata
+    def test_char_length(self, metadata, connection):
         t1 = Table(
             "t1",
             metadata,
@@ -847,40 +840,38 @@ class TypesTest(fixtures.TestBase):
             Column("c3", CHAR(200)),
             Column("c4", NCHAR(180)),
         )
-        t1.create()
-        m2 = MetaData(testing.db)
-        t2 = Table("t1", m2, autoload=True)
+        t1.create(connection)
+        m2 = MetaData()
+        t2 = Table("t1", m2, autoload_with=connection)
         eq_(t2.c.c1.type.length, 50)
         eq_(t2.c.c2.type.length, 250)
         eq_(t2.c.c3.type.length, 200)
         eq_(t2.c.c4.type.length, 180)
 
-    @testing.provide_metadata
-    def test_long_type(self):
-        metadata = self.metadata
+    def test_long_type(self, metadata, connection):
 
         t = Table("t", metadata, Column("data", oracle.LONG))
-        metadata.create_all(testing.db)
-        testing.db.execute(t.insert(), data="xyz")
-        eq_(testing.db.scalar(select([t.c.data])), "xyz")
+        metadata.create_all(connection)
+        connection.execute(t.insert(), dict(data="xyz"))
+        eq_(connection.scalar(select(t.c.data)), "xyz")
 
-    def test_longstring(self):
-        metadata = MetaData(testing.db)
-        testing.db.execute(
+    def test_longstring(self, metadata, connection):
+        exec_sql(
+            connection,
             """
         CREATE TABLE Z_TEST
         (
           ID        NUMERIC(22) PRIMARY KEY,
           ADD_USER  VARCHAR2(20)  NOT NULL
         )
-        """
+        """,
         )
         try:
-            t = Table("z_test", metadata, autoload=True)
-            t.insert().execute(id=1.0, add_user="foobar")
-            assert t.select().execute().fetchall() == [(1, "foobar")]
+            t = Table("z_test", metadata, autoload_with=connection)
+            connection.execute(t.insert(), dict(id=1.0, add_user="foobar"))
+            assert connection.execute(t.select()).fetchall() == [(1, "foobar")]
         finally:
-            testing.db.execute("DROP TABLE Z_TEST")
+            exec_sql(connection, "DROP TABLE Z_TEST")
 
 
 class LOBFetchTest(fixtures.TablesTest):
@@ -908,7 +899,7 @@ class LOBFetchTest(fixtures.TablesTest):
         )
 
     @classmethod
-    def insert_data(cls):
+    def insert_data(cls, connection):
         cls.data = data = [
             dict(
                 id=i,
@@ -918,7 +909,7 @@ class LOBFetchTest(fixtures.TablesTest):
             for i in range(1, 20)
         ]
 
-        testing.db.execute(cls.tables.z_test.insert(), data)
+        connection.execute(cls.tables.z_test.insert(), data)
 
         binary_table = cls.tables.binary_table
         fname = os.path.join(
@@ -928,32 +919,34 @@ class LOBFetchTest(fixtures.TablesTest):
             cls.stream = stream = file_.read(12000)
 
         for i in range(1, 11):
-            binary_table.insert().execute(id=i, data=stream)
+            connection.execute(binary_table.insert(), dict(id=i, data=stream))
 
     def test_lobs_without_convert(self):
         engine = testing_engine(options=dict(auto_convert_lobs=False))
         t = self.tables.z_test
-        row = engine.execute(t.select().where(t.c.id == 1)).first()
-        eq_(row["data"].read(), "this is text 1")
-        eq_(row["bindata"].read(), b("this is binary 1"))
+        with engine.begin() as conn:
+            row = conn.execute(t.select().where(t.c.id == 1)).first()
+            eq_(row._mapping["data"].read(), "this is text 1")
+            eq_(row._mapping["bindata"].read(), b("this is binary 1"))
 
-    def test_lobs_with_convert(self):
+    def test_lobs_with_convert(self, connection):
         t = self.tables.z_test
-        row = testing.db.execute(t.select().where(t.c.id == 1)).first()
-        eq_(row["data"], "this is text 1")
-        eq_(row["bindata"], b("this is binary 1"))
+        row = connection.execute(t.select().where(t.c.id == 1)).first()
+        eq_(row._mapping["data"], "this is text 1")
+        eq_(row._mapping["bindata"], b("this is binary 1"))
 
-    def test_lobs_with_convert_raw(self):
-        row = testing.db.execute("select data, bindata from z_test").first()
-        eq_(row["data"], "this is text 1")
-        eq_(row["bindata"], b("this is binary 1"))
+    def test_lobs_with_convert_raw(self, connection):
+        row = exec_sql(connection, "select data, bindata from z_test").first()
+        eq_(row._mapping["data"], "this is text 1")
+        eq_(row._mapping["bindata"], b("this is binary 1"))
 
     def test_lobs_without_convert_many_rows(self):
         engine = testing_engine(
             options=dict(auto_convert_lobs=False, arraysize=1)
         )
-        result = engine.execute(
-            "select id, data, bindata from z_test order by id"
+        result = exec_sql(
+            engine.connect(),
+            "select id, data, bindata from z_test order by id",
         )
         results = result.fetchall()
 
@@ -961,9 +954,9 @@ class LOBFetchTest(fixtures.TablesTest):
             eq_(
                 [
                     dict(
-                        id=row["id"],
-                        data=row["data"].read(),
-                        bindata=row["bindata"].read(),
+                        id=row._mapping["id"],
+                        data=row._mapping["data"].read(),
+                        bindata=row._mapping["bindata"].read(),
                     )
                     for row in results
                 ],
@@ -986,36 +979,105 @@ class LOBFetchTest(fixtures.TablesTest):
         engine = testing_engine(
             options=dict(auto_convert_lobs=True, arraysize=1)
         )
-        result = engine.execute(
-            "select id, data, bindata from z_test order by id"
-        )
-        results = result.fetchall()
+        with engine.connect() as conn:
+            result = exec_sql(
+                conn,
+                "select id, data, bindata from z_test order by id",
+            )
+            results = result.fetchall()
 
-        eq_(
-            [
-                dict(id=row["id"], data=row["data"], bindata=row["bindata"])
-                for row in results
-            ],
-            self.data,
+            eq_(
+                [
+                    dict(
+                        id=row._mapping["id"],
+                        data=row._mapping["data"],
+                        bindata=row._mapping["bindata"],
+                    )
+                    for row in results
+                ],
+                self.data,
+            )
+
+    @testing.combinations(
+        (UnicodeText(),), (Text(),), (LargeBinary(),), argnames="datatype"
+    )
+    @testing.combinations((10,), (100,), (250,), argnames="datasize")
+    @testing.combinations(
+        ("x,y,z"), ("y"), ("y,x,z"), ("x,z,y"), argnames="retcols"
+    )
+    def test_insert_returning_w_lobs(
+        self, datatype, datasize, retcols, metadata, connection
+    ):
+        long_text = Table(
+            "long_text",
+            metadata,
+            Column("x", Integer),
+            Column("y", datatype),
+            Column("z", Integer),
+        )
+        long_text.create(connection)
+
+        if isinstance(datatype, UnicodeText):
+            word_seed = u"ab🐍’«cdefg"
+        else:
+            word_seed = "abcdef"
+
+        some_text = u" ".join(
+            "".join(random.choice(word_seed) for j in range(150))
+            for i in range(datasize)
+        )
+        if isinstance(datatype, LargeBinary):
+            some_text = some_text.encode("ascii")
+
+        data = {"x": 5, "y": some_text, "z": 10}
+        return_columns = [long_text.c[col] for col in retcols.split(",")]
+        expected = tuple(data[col] for col in retcols.split(","))
+        result = connection.execute(
+            long_text.insert().returning(*return_columns),
+            data,
         )
 
-    def test_large_stream(self):
+        eq_(result.fetchall(), [expected])
+
+    def test_insert_returning_w_unicode(self, metadata, connection):
+        long_text = Table(
+            "long_text",
+            metadata,
+            Column("x", Integer),
+            Column("y", Unicode(255)),
+        )
+        long_text.create(connection)
+
+        word_seed = u"ab🐍’«cdefg"
+
+        some_text = u" ".join(
+            "".join(random.choice(word_seed) for j in range(10))
+            for i in range(15)
+        )
+
+        data = {"x": 5, "y": some_text}
+        result = connection.execute(
+            long_text.insert().returning(long_text.c.y),
+            data,
+        )
+
+        eq_(result.fetchall(), [(some_text,)])
+
+    def test_large_stream(self, connection):
         binary_table = self.tables.binary_table
-        result = (
-            binary_table.select()
-            .order_by(binary_table.c.id)
-            .execute()
-            .fetchall()
-        )
+        result = connection.execute(
+            binary_table.select().order_by(binary_table.c.id)
+        ).fetchall()
         eq_(result, [(i, self.stream) for i in range(1, 11)])
 
     def test_large_stream_single_arraysize(self):
         binary_table = self.tables.binary_table
         eng = testing_engine(options={"arraysize": 1})
-        result = eng.execute(
-            binary_table.select().order_by(binary_table.c.id)
-        ).fetchall()
-        eq_(result, [(i, self.stream) for i in range(1, 11)])
+        with eng.connect() as conn:
+            result = conn.execute(
+                binary_table.select().order_by(binary_table.c.id)
+            ).fetchall()
+            eq_(result, [(i, self.stream) for i in range(1, 11)])
 
 
 class EuroNumericTest(fixtures.TestBase):
@@ -1026,7 +1088,7 @@ class EuroNumericTest(fixtures.TestBase):
     __only_on__ = "oracle+cx_oracle"
     __backend__ = True
 
-    def setup(self):
+    def setup_test(self):
         connect = testing.db.pool._creator
 
         def _creator():
@@ -1038,7 +1100,7 @@ class EuroNumericTest(fixtures.TestBase):
 
         self.engine = testing_engine(options={"creator": _creator})
 
-    def teardown(self):
+    def teardown_test(self):
         self.engine.dispose()
 
     def test_were_getting_a_comma(self):
@@ -1091,7 +1153,10 @@ class EuroNumericTest(fixtures.TestBase):
                     {},
                 ),
             ]:
-                test_exp = conn.scalar(stmt, **kw)
+                if isinstance(stmt, util.string_types):
+                    test_exp = conn.exec_driver_sql(stmt, kw).scalar()
+                else:
+                    test_exp = conn.scalar(stmt, **kw)
                 eq_(test_exp, exp)
                 assert type(test_exp) is type(exp)
 
@@ -1116,10 +1181,10 @@ class SetInputSizesTest(fixtures.TestBase):
         (CHAR(30), "test", "FIXED_CHAR", False),
         (NCHAR(30), u("test"), "FIXED_NCHAR", False),
         (oracle.LONG(), "test", None, False),
+        argnames="datatype, value, sis_value_text, set_nchar_flag",
     )
-    @testing.provide_metadata
     def test_setinputsizes(
-        self, datatype, value, sis_value_text, set_nchar_flag
+        self, metadata, datatype, value, sis_value_text, set_nchar_flag
     ):
         if isinstance(sis_value_text, str):
             sis_value = getattr(testing.db.dialect.dbapi, sis_value_text)
@@ -1128,6 +1193,7 @@ class SetInputSizesTest(fixtures.TestBase):
 
         class TestTypeDec(TypeDecorator):
             impl = NullType()
+            cache_ok = True
 
             def load_dialect_impl(self, dialect):
                 if dialect.name == "oracle":
@@ -1135,7 +1201,7 @@ class SetInputSizesTest(fixtures.TestBase):
                 else:
                     return self.impl
 
-        m = self.metadata
+        m = metadata
         # Oracle can have only one column of type LONG so we make three
         # tables rather than one table w/ three columns
         t1 = Table("t1", m, Column("foo", datatype))
@@ -1143,7 +1209,6 @@ class SetInputSizesTest(fixtures.TestBase):
             "t2", m, Column("foo", NullType().with_variant(datatype, "oracle"))
         )
         t3 = Table("t3", m, Column("foo", TestTypeDec()))
-        m.create_all()
 
         class CursorWrapper(object):
             # cx_oracle cursor can't be modified so we have to
@@ -1167,6 +1232,10 @@ class SetInputSizesTest(fixtures.TestBase):
             engine = testing.db
 
         with engine.connect() as conn:
+            conn.begin()
+
+            m.create_all(conn, checkfirst=False)
+
             connection_fairy = conn.connection
             for tab in [t1, t2, t3]:
                 with mock.patch.object(
@@ -1187,7 +1256,7 @@ class SetInputSizesTest(fixtures.TestBase):
                         [mock.call.setinputsizes()],
                     )
 
-    def test_event_no_native_float(self):
+    def test_event_no_native_float(self, metadata):
         def _remove_type(inputsizes, cursor, statement, parameters, context):
             for param, dbapitype in list(inputsizes.items()):
                 if dbapitype is testing.db.dialect.dbapi.NATIVE_FLOAT:
@@ -1195,6 +1264,8 @@ class SetInputSizesTest(fixtures.TestBase):
 
         event.listen(testing.db, "do_setinputsizes", _remove_type)
         try:
-            self.test_setinputsizes(oracle.BINARY_FLOAT, 25.34534, None, False)
+            self.test_setinputsizes(
+                metadata, oracle.BINARY_FLOAT, 25.34534, None, False
+            )
         finally:
             event.remove(testing.db, "do_setinputsizes", _remove_type)
