@@ -1,6 +1,6 @@
 """Exercises for eager loading.
 
-Derived from mailing list-reported problems and trac tickets.
+Derived from mailing list-reported problems and issue tracker issues.
 
 These are generally very old 0.1-era tests and at some point should
 be cleaned up and modernized.
@@ -15,11 +15,11 @@ from sqlalchemy import String
 from sqlalchemy import testing
 from sqlalchemy import text
 from sqlalchemy.orm import backref
-from sqlalchemy.orm import create_session
-from sqlalchemy.orm import mapper
 from sqlalchemy.orm import relationship
+from sqlalchemy.orm import Session
 from sqlalchemy.testing import eq_
 from sqlalchemy.testing import fixtures
+from sqlalchemy.testing.fixtures import fixture_session
 from sqlalchemy.testing.schema import Column
 from sqlalchemy.testing.schema import Table
 
@@ -111,11 +111,11 @@ class EagerTest(fixtures.MappedTest):
             cls.tables.categories,
         )
 
-        mapper(Owner, owners)
+        cls.mapper_registry.map_imperatively(Owner, owners)
 
-        mapper(Category, categories)
+        cls.mapper_registry.map_imperatively(Category, categories)
 
-        mapper(
+        cls.mapper_registry.map_imperatively(
             Option,
             options,
             properties=dict(
@@ -124,7 +124,7 @@ class EagerTest(fixtures.MappedTest):
             ),
         )
 
-        mapper(
+        cls.mapper_registry.map_imperatively(
             Thing,
             tests,
             properties=dict(
@@ -143,7 +143,7 @@ class EagerTest(fixtures.MappedTest):
         )
 
     @classmethod
-    def insert_data(cls):
+    def insert_data(cls, connection):
         Owner, Category, Option, Thing = (
             cls.classes.Owner,
             cls.classes.Category,
@@ -151,7 +151,7 @@ class EagerTest(fixtures.MappedTest):
             cls.classes.Thing,
         )
 
-        session = create_session()
+        session = Session(connection)
 
         o = Owner()
         c = Category(name="Some Category")
@@ -167,7 +167,7 @@ class EagerTest(fixtures.MappedTest):
 
         session.flush()
 
-    def test_noorm(self):
+    def test_noorm(self, connection):
         """test the control case"""
 
         tests, options, categories = (
@@ -187,30 +187,28 @@ class EagerTest(fixtures.MappedTest):
 
         # not orm style correct query
         print("Obtaining correct results without orm")
-        result = (
-            sa.select(
-                [tests.c.id, categories.c.name],
+        result = connection.execute(
+            sa.select(tests.c.id, categories.c.name)
+            .where(
                 sa.and_(
                     tests.c.owner_id == 1,
                     sa.or_(
                         options.c.someoption == None,  # noqa
                         options.c.someoption == False,
                     ),
-                ),
-                order_by=[tests.c.id],
-                from_obj=[
-                    tests.join(categories).outerjoin(
-                        options,
-                        sa.and_(
-                            tests.c.id == options.c.test_id,
-                            tests.c.owner_id == options.c.owner_id,
-                        ),
-                    )
-                ],
+                )
             )
-            .execute()
-            .fetchall()
-        )
+            .order_by(tests.c.id)
+            .select_from(
+                tests.join(categories).outerjoin(
+                    options,
+                    sa.and_(
+                        tests.c.id == options.c.test_id,
+                        tests.c.owner_id == options.c.owner_id,
+                    ),
+                )
+            )
+        ).fetchall()
         eq_(result, [(1, "Some Category"), (3, "Some Category")])
 
     def test_withoutjoinedload(self):
@@ -220,7 +218,7 @@ class EagerTest(fixtures.MappedTest):
             self.tables.options,
         )
 
-        s = create_session()
+        s = fixture_session()
         result = (
             s.query(Thing)
             .select_from(
@@ -260,8 +258,8 @@ class EagerTest(fixtures.MappedTest):
             self.tables.options,
         )
 
-        s = create_session()
-        q = s.query(Thing).options(sa.orm.joinedload("category"))
+        s = fixture_session()
+        q = s.query(Thing).options(sa.orm.joinedload(Thing.category))
 
         result = q.select_from(
             tests.outerjoin(
@@ -275,8 +273,8 @@ class EagerTest(fixtures.MappedTest):
             sa.and_(
                 tests.c.owner_id == 1,
                 sa.or_(
-                    options.c.someoption == None,  # noqa
-                    options.c.someoption == False,
+                    options.c.someoption == None,
+                    options.c.someoption == False,  # noqa
                 ),
             )
         )
@@ -293,17 +291,17 @@ class EagerTest(fixtures.MappedTest):
             self.tables.options,
         )
 
-        s = create_session()
-        q = s.query(Thing).options(sa.orm.joinedload("category"))
+        s = fixture_session()
+        q = s.query(Thing).options(sa.orm.joinedload(Thing.category))
         result = q.filter(
             sa.and_(
                 tests.c.owner_id == 1,
                 sa.or_(
-                    options.c.someoption == None,  # noqa
-                    options.c.someoption == False,
+                    options.c.someoption == None,
+                    options.c.someoption == False,  # noqa
                 ),
             )
-        ).outerjoin("owner_option")
+        ).outerjoin(Thing.owner_option)
 
         result_str = ["%d %s" % (t.id, t.category.name) for t in result]
         eq_(result_str, ["1 Some Category", "3 Some Category"])
@@ -312,14 +310,14 @@ class EagerTest(fixtures.MappedTest):
     def test_without_outerjoin_literal(self):
         Thing, tests = (self.classes.Thing, self.tables.tests)
 
-        s = create_session()
-        q = s.query(Thing).options(sa.orm.joinedload("category"))
+        s = fixture_session()
+        q = s.query(Thing).options(sa.orm.joinedload(Thing.category))
         result = q.filter(
             (tests.c.owner_id == 1)
             & text(
                 "options.someoption is null or options.someoption=:opt"
             ).bindparams(opt=False)
-        ).join("owner_option")
+        ).join(Thing.owner_option)
 
         result_str = ["%d %s" % (t.id, t.category.name) for t in result]
         eq_(result_str, ["3 Some Category"])
@@ -331,15 +329,15 @@ class EagerTest(fixtures.MappedTest):
             self.tables.options,
         )
 
-        s = create_session()
-        q = s.query(Thing).options(sa.orm.joinedload("category"))
+        s = fixture_session()
+        q = s.query(Thing).options(sa.orm.joinedload(Thing.category))
         result = q.filter(
             (tests.c.owner_id == 1)
             & (
                 (options.c.someoption == None)
                 | (options.c.someoption == False)
             )  # noqa
-        ).join("owner_option")
+        ).join(Thing.owner_option)
 
         result_str = ["%d %s" % (t.id, t.category.name) for t in result]
         eq_(result_str, ["3 Some Category"])
@@ -397,9 +395,9 @@ class EagerTest2(fixtures.MappedTest):
         )
 
         # set up bi-directional eager loads
-        mapper(Left, left)
-        mapper(Right, right)
-        mapper(
+        cls.mapper_registry.map_imperatively(Left, left)
+        cls.mapper_registry.map_imperatively(Right, right)
+        cls.mapper_registry.map_imperatively(
             Middle,
             middle,
             properties=dict(
@@ -434,11 +432,11 @@ class EagerTest2(fixtures.MappedTest):
         p.left.append(Left("l1"))
         p.right.append(Right("r1"))
 
-        session = create_session()
+        session = fixture_session()
         session.add(p)
         session.flush()
         session.expunge_all()
-        obj = session.query(Left).filter_by(data="l1").one()
+        session.query(Left).filter_by(data="l1").one()
 
 
 class EagerTest3(fixtures.MappedTest):
@@ -496,8 +494,8 @@ class EagerTest3(fixtures.MappedTest):
             self.tables.datas,
         )
 
-        mapper(Data, datas)
-        mapper(
+        self.mapper_registry.map_imperatively(Data, datas)
+        self.mapper_registry.map_imperatively(
             Foo,
             foo,
             properties={
@@ -507,9 +505,11 @@ class EagerTest3(fixtures.MappedTest):
             },
         )
 
-        mapper(Stat, stats, properties={"data": relationship(Data)})
+        self.mapper_registry.map_imperatively(
+            Stat, stats, properties={"data": relationship(Data)}
+        )
 
-        session = create_session()
+        session = fixture_session()
 
         data = [Data(a=x) for x in range(5)]
         session.add_all(data)
@@ -530,19 +530,21 @@ class EagerTest3(fixtures.MappedTest):
         )
         session.flush()
 
-        arb_data = sa.select(
-            [stats.c.data_id, sa.func.max(stats.c.somedata).label("max")],
-            stats.c.data_id <= 5,
-            group_by=[stats.c.data_id],
+        arb_data = (
+            sa.select(
+                stats.c.data_id, sa.func.max(stats.c.somedata).label("max")
+            )
+            .where(stats.c.data_id <= 5)
+            .group_by(stats.c.data_id)
         )
 
-        arb_result = arb_data.execute().fetchall()
+        arb_result = session.connection().execute(arb_data).fetchall()
 
         # order the result list descending based on 'max'
-        arb_result.sort(key=lambda a: a["max"], reverse=True)
+        arb_result.sort(key=lambda a: a._mapping["max"], reverse=True)
 
         # extract just the "data_id" from it
-        arb_result = [row["data_id"] for row in arb_result]
+        arb_result = [row._mapping["data_id"] for row in arb_result]
 
         arb_data = arb_data.alias("arb")
 
@@ -550,7 +552,7 @@ class EagerTest3(fixtures.MappedTest):
         # "order by max desc" separately
         q = (
             session.query(Data)
-            .options(sa.orm.joinedload("foo"))
+            .options(sa.orm.joinedload(Data.foo))
             .select_from(
                 datas.join(arb_data, arb_data.c.data_id == datas.c.id)
             )
@@ -612,8 +614,8 @@ class EagerTest4(fixtures.MappedTest):
             self.tables.departments,
         )
 
-        mapper(Employee, employees)
-        mapper(
+        self.mapper_registry.map_imperatively(Employee, employees)
+        self.mapper_registry.map_imperatively(
             Department,
             departments,
             properties=dict(
@@ -631,13 +633,13 @@ class EagerTest4(fixtures.MappedTest):
         for e in "Joe", "Bob", "Mary", "Wally":
             d2.employees.append(Employee(name=e))
 
-        sess = create_session()
+        sess = fixture_session()
         sess.add_all((d1, d2))
         sess.flush()
 
         q = (
             sess.query(Department)
-            .join("employees")
+            .join(Department.employees)
             .filter(Employee.name.startswith("J"))
             .distinct()
             .order_by(sa.desc(Department.name))
@@ -733,9 +735,9 @@ class EagerTest5(fixtures.MappedTest):
             self.tables.derivedII,
         )
 
-        commentMapper = mapper(Comment, comments)
+        self.mapper_registry.map_imperatively(Comment, comments)
 
-        baseMapper = mapper(
+        baseMapper = self.mapper_registry.map_imperatively(
             Base,
             base,
             properties=dict(
@@ -745,11 +747,15 @@ class EagerTest5(fixtures.MappedTest):
             ),
         )
 
-        mapper(Derived, derived, inherits=baseMapper)
+        self.mapper_registry.map_imperatively(
+            Derived, derived, inherits=baseMapper
+        )
 
-        mapper(DerivedII, derivedII, inherits=baseMapper)
+        self.mapper_registry.map_imperatively(
+            DerivedII, derivedII, inherits=baseMapper
+        )
 
-        sess = create_session()
+        sess = fixture_session()
         d = Derived("uid1", "x", "y")
         d.comments = [Comment("uid1", "comment")]
         d2 = DerivedII("uid2", "xx", "z")
@@ -761,7 +767,7 @@ class EagerTest5(fixtures.MappedTest):
         # this eager load sets up an AliasedClauses for the "comment"
         # relationship, then stores it in clauses_by_lead_mapper[mapper for
         # Derived]
-        d = sess.query(Derived).get("uid1")
+        d = sess.get(Derived, "uid1")
         sess.expunge_all()
         assert len([c for c in d.comments]) == 1
 
@@ -769,7 +775,7 @@ class EagerTest5(fixtures.MappedTest):
         # relationship, and should store it in clauses_by_lead_mapper[mapper
         # for DerivedII].  the bug was that the previous AliasedClause create
         # prevented this population from occurring.
-        d2 = sess.query(DerivedII).get("uid2")
+        d2 = sess.get(DerivedII, "uid2")
         sess.expunge_all()
 
         # object is not in the session; therefore the lazy load cant trigger
@@ -872,15 +878,15 @@ class EagerTest6(fixtures.MappedTest):
             self.classes.InheritedPart,
         )
 
-        p_m = mapper(Part, parts)
+        p_m = self.mapper_registry.map_imperatively(Part, parts)
 
-        mapper(
+        self.mapper_registry.map_imperatively(
             InheritedPart,
             inherited_part,
             properties=dict(part=relationship(Part, lazy="joined")),
         )
 
-        d_m = mapper(
+        d_m = self.mapper_registry.map_imperatively(
             Design,
             design,
             properties=dict(
@@ -892,7 +898,7 @@ class EagerTest6(fixtures.MappedTest):
             ),
         )
 
-        mapper(DesignType, design_types)
+        self.mapper_registry.map_imperatively(DesignType, design_types)
 
         d_m.add_property(
             "type", relationship(DesignType, lazy="joined", backref="designs")
@@ -908,11 +914,11 @@ class EagerTest6(fixtures.MappedTest):
         )
 
         d = Design()
-        sess = create_session()
+        sess = fixture_session()
         sess.add(d)
         sess.flush()
         sess.expunge_all()
-        x = sess.query(Design).get(1)
+        x = sess.get(Design, 1)
         x.inheritedParts
 
 
@@ -1003,15 +1009,15 @@ class EagerTest7(fixtures.MappedTest):
             self.classes.Address,
         )
 
-        mapper(Address, addresses)
+        self.mapper_registry.map_imperatively(Address, addresses)
 
-        mapper(
+        self.mapper_registry.map_imperatively(
             Company,
             companies,
             properties={"addresses": relationship(Address, lazy="joined")},
         )
 
-        mapper(
+        self.mapper_registry.map_imperatively(
             Invoice,
             invoices,
             properties={"company": relationship(Company, lazy="joined")},
@@ -1022,7 +1028,7 @@ class EagerTest7(fixtures.MappedTest):
         c1 = Company(company_name="company 1", addresses=[a1, a2])
         i1 = Invoice(date=datetime.datetime.now(), company=c1)
 
-        session = create_session()
+        session = fixture_session()
         session.add(i1)
         session.flush()
 
@@ -1030,10 +1036,10 @@ class EagerTest7(fixtures.MappedTest):
         invoice_id = i1.invoice_id
 
         session.expunge_all()
-        c = session.query(Company).get(company_id)
+        c = session.get(Company, company_id)
 
         session.expunge_all()
-        i = session.query(Invoice).get(invoice_id)
+        i = session.get(Invoice, invoice_id)
 
         def go():
             eq_(c, i.company)
@@ -1135,11 +1141,10 @@ class EagerTest8(fixtures.MappedTest):
             pass
 
     def test_nested_joins(self):
-        task, Task_Type, Joined, prj, task_type, msg = (
+        task, Task_Type, Joined, task_type, msg = (
             self.tables.task,
             self.classes.Task_Type,
             self.classes.Joined,
-            self.tables.prj,
             self.tables.task_type,
             self.tables.msg,
         )
@@ -1148,31 +1153,34 @@ class EagerTest8(fixtures.MappedTest):
         # concerning corresponding_column() being extremely accurate
         # as well as how mapper sets up its column properties
 
-        mapper(Task_Type, task_type)
-
-        tsk_cnt_join = sa.outerjoin(prj, task, task.c.prj_id == prj.c.id)
+        self.mapper_registry.map_imperatively(Task_Type, task_type)
 
         j = sa.outerjoin(task, msg, task.c.id == msg.c.task_id)
-        jj = sa.select(
-            [
+        jj = (
+            sa.select(
                 task.c.id.label("task_id"),
                 sa.func.count(msg.c.id).label("props_cnt"),
-            ],
-            from_obj=[j],
-            group_by=[task.c.id],
-        ).alias("prop_c_s")
+            )
+            .select_from(j)
+            .group_by(task.c.id)
+            .alias("prop_c_s")
+        )
         jjj = sa.join(task, jj, task.c.id == jj.c.task_id)
 
-        mapper(
+        self.mapper_registry.map_imperatively(
             Joined,
             jjj,
             properties=dict(type=relationship(Task_Type, lazy="joined")),
         )
 
-        session = create_session()
+        session = fixture_session()
 
         eq_(
-            session.query(Joined).limit(10).offset(0).one(),
+            session.query(Joined)
+            .order_by(Joined.id)
+            .limit(10)
+            .offset(0)
+            .one(),
             Joined(id=1, title="task 1", props_cnt=0),
         )
 
@@ -1252,11 +1260,11 @@ class EagerTest9(fixtures.MappedTest):
             cls.classes.Entry,
         )
 
-        mapper(Account, accounts)
+        cls.mapper_registry.map_imperatively(Account, accounts)
 
-        mapper(Transaction, transactions)
+        cls.mapper_registry.map_imperatively(Transaction, transactions)
 
-        mapper(
+        cls.mapper_registry.map_imperatively(
             Entry,
             entries,
             properties=dict(
@@ -1284,18 +1292,18 @@ class EagerTest9(fixtures.MappedTest):
             self.classes.Transaction,
         )
 
-        session = create_session()
+        session = fixture_session()
 
         tx1 = Transaction(name="tx1")
         tx2 = Transaction(name="tx2")
 
         acc1 = Account(name="acc1")
-        ent11 = Entry(name="ent11", account=acc1, transaction=tx1)
-        ent12 = Entry(name="ent12", account=acc1, transaction=tx2)
+        Entry(name="ent11", account=acc1, transaction=tx1)
+        Entry(name="ent12", account=acc1, transaction=tx2)
 
         acc2 = Account(name="acc2")
-        ent21 = Entry(name="ent21", account=acc2, transaction=tx1)
-        ent22 = Entry(name="ent22", account=acc2, transaction=tx2)
+        Entry(name="ent21", account=acc2, transaction=tx1)
+        Entry(name="ent22", account=acc2, transaction=tx2)
 
         session.add(acc1)
         session.flush()
@@ -1309,9 +1317,10 @@ class EagerTest9(fixtures.MappedTest):
             acc = (
                 session.query(Account)
                 .options(
-                    sa.orm.joinedload_all(
-                        "entries.transaction.entries.account"
-                    )
+                    sa.orm.joinedload(Account.entries)
+                    .joinedload(Entry.transaction)
+                    .joinedload(Transaction.entries)
+                    .joinedload(Entry.account)
                 )
                 .order_by(Account.account_id)
             ).first()
