@@ -1,17 +1,15 @@
 """test the current state of the hasparent() flag."""
 
-
 from sqlalchemy import ForeignKey
 from sqlalchemy import Integer
 from sqlalchemy import testing
 from sqlalchemy.orm import attributes
 from sqlalchemy.orm import exc as orm_exc
-from sqlalchemy.orm import mapper
 from sqlalchemy.orm import relationship
-from sqlalchemy.orm import Session
 from sqlalchemy.testing import assert_raises_message
 from sqlalchemy.testing import eq_
 from sqlalchemy.testing import fixtures
+from sqlalchemy.testing.fixtures import fixture_session
 from sqlalchemy.testing.schema import Column
 from sqlalchemy.testing.schema import Table
 from sqlalchemy.testing.util import gc_collect
@@ -63,8 +61,10 @@ class ParentRemovalTest(fixtures.MappedTest):
 
     @classmethod
     def setup_mappers(cls):
-        mapper(cls.classes.Address, cls.tables.addresses)
-        mapper(
+        cls.mapper_registry.map_imperatively(
+            cls.classes.Address, cls.tables.addresses
+        )
+        cls.mapper_registry.map_imperatively(
             cls.classes.User,
             cls.tables.users,
             properties={
@@ -83,7 +83,7 @@ class ParentRemovalTest(fixtures.MappedTest):
     def _fixture(self):
         User, Address = self.classes.User, self.classes.Address
 
-        s = Session()
+        s = fixture_session()
 
         u1 = User()
         a1 = Address()
