@@ -1,18 +1,22 @@
 """test the inspection registry system."""
 
 from sqlalchemy import exc
+from sqlalchemy import ForeignKey
 from sqlalchemy import inspect
 from sqlalchemy import testing
 from sqlalchemy.orm import aliased
 from sqlalchemy.orm import class_mapper
+from sqlalchemy.orm import relationship
 from sqlalchemy.orm import Session
 from sqlalchemy.orm import synonym
 from sqlalchemy.orm.attributes import instance_state
 from sqlalchemy.orm.attributes import NO_VALUE
+from sqlalchemy.orm.base import InspectionAttr
 from sqlalchemy.orm.util import identity_key
 from sqlalchemy.testing import assert_raises_message
 from sqlalchemy.testing import eq_
 from sqlalchemy.testing import is_
+from sqlalchemy.testing.fixtures import fixture_session
 from test.orm import _fixtures
 
 
@@ -46,11 +50,11 @@ class TestORMInspection(_fixtures.FixtureTest):
         insp = inspect(User)
         is_(insp.local_table, user_table)
 
-    def test_mapped_table(self):
+    def test_persist_selectable(self):
         User = self.classes.User
         user_table = self.tables.users
         insp = inspect(User)
-        is_(insp.mapped_table, user_table)
+        is_(insp.persist_selectable, user_table)
 
     def test_mapper_selectable(self):
         User = self.classes.User
@@ -61,8 +65,6 @@ class TestORMInspection(_fixtures.FixtureTest):
         assert not insp.is_aliased_class
 
     def test_mapper_selectable_fixed(self):
-        from sqlalchemy.orm import mapper
-
         class Foo(object):
             pass
 
@@ -71,8 +73,10 @@ class TestORMInspection(_fixtures.FixtureTest):
 
         user_table = self.tables.users
         addresses_table = self.tables.addresses
-        mapper(Foo, user_table, with_polymorphic=(Bar,))
-        mapper(
+        self.mapper_registry.map_imperatively(
+            Foo, user_table, with_polymorphic=(Bar,)
+        )
+        self.mapper_registry.map_imperatively(
             Bar,
             addresses_table,
             inherits=Foo,
@@ -240,7 +244,6 @@ class TestORMInspection(_fixtures.FixtureTest):
             HYBRID_METHOD,
         )
         from sqlalchemy import Table, MetaData, Integer, Column
-        from sqlalchemy.orm import mapper
         from sqlalchemy.orm.interfaces import NOT_EXTENSION
 
         class SomeClass(self.classes.User):
@@ -254,6 +257,9 @@ class TestORMInspection(_fixtures.FixtureTest):
             def conv(self, fn):
                 raise NotImplementedError()
 
+        class Address(self.classes.Address):
+            pass
+
         class SomeSubClass(SomeClass):
             @hybrid_property
             def upper_name(self):
@@ -263,11 +269,19 @@ class TestORMInspection(_fixtures.FixtureTest):
             def foo(self):
                 raise NotImplementedError()
 
-        t = Table(
-            "sometable", MetaData(), Column("id", Integer, primary_key=True)
+        m = MetaData()
+        t = Table("sometable", m, Column("id", Integer, primary_key=True))
+        ta = Table(
+            "address_t",
+            m,
+            Column("id", Integer, primary_key=True),
+            Column("s_id", ForeignKey("sometable.id")),
         )
-        mapper(SomeClass, t)
-        mapper(SomeSubClass, inherits=SomeClass)
+        self.mapper_registry.map_imperatively(
+            SomeClass, t, properties={"addresses": relationship(Address)}
+        )
+        self.mapper_registry.map_imperatively(Address, ta)
+        self.mapper_registry.map_imperatively(SomeSubClass, inherits=SomeClass)
 
         insp = inspect(SomeSubClass)
         eq_(
@@ -291,7 +305,7 @@ class TestORMInspection(_fixtures.FixtureTest):
             insp.all_orm_descriptors.upper_name,
             SomeSubClass.__dict__["upper_name"],
         )
-        is_(insp.all_orm_descriptors.some_assoc, SomeClass.some_assoc)
+        is_(insp.all_orm_descriptors.some_assoc, SomeClass.some_assoc.parent)
         is_(
             inspect(SomeClass).all_orm_descriptors.upper_name,
             SomeClass.__dict__["upper_name"],
@@ -303,6 +317,13 @@ class TestORMInspection(_fixtures.FixtureTest):
         insp = inspect(u1)
         is_(insp, instance_state(u1))
 
+    def test_instance_state_info(self):
+        User = self.classes.User
+        u1 = User()
+        insp = inspect(u1)
+        insp.info["some_key"] = "value"
+        eq_(inspect(u1).info["some_key"], "value")
+
     def test_instance_state_attr(self):
         User = self.classes.User
         u1 = User(name="ed")
@@ -334,8 +355,8 @@ class TestORMInspection(_fixtures.FixtureTest):
         eq_(insp.attrs.addresses.loaded_value, NO_VALUE)
         # regular accessor sets it
         eq_(insp.attrs.addresses.value, [])
-        # now the None is there
-        eq_(insp.attrs.addresses.loaded_value, [])
+        # stays as NO_VALUE, this is #4519
+        eq_(insp.attrs.addresses.loaded_value, NO_VALUE)
 
     def test_instance_state_collection_attr_hist(self):
         User = self.classes.User
@@ -345,12 +366,13 @@ class TestORMInspection(_fixtures.FixtureTest):
         eq_(hist.unchanged, None)
         u1.addresses
         hist = insp.attrs.addresses.history
-        eq_(hist.unchanged, [])
+        # stays, this is #4519
+        eq_(hist.unchanged, None)
 
     def test_instance_state_scalar_attr_hist(self):
         User = self.classes.User
         u1 = User(name="ed")
-        sess = Session()
+        sess = fixture_session()
         sess.add(u1)
         sess.commit()
         assert "name" not in u1.__dict__
@@ -367,12 +389,13 @@ class TestORMInspection(_fixtures.FixtureTest):
         eq_(hist.unchanged, ())
         u1.addresses
         hist = insp.attrs.addresses.load_history()
-        eq_(hist.unchanged, [])
+        # stays, this is #4519
+        eq_(hist.unchanged, ())
 
     def test_instance_state_scalar_attr_hist_load(self):
         User = self.classes.User
         u1 = User(name="ed")
-        sess = Session()
+        sess = fixture_session()
         sess.add(u1)
         sess.commit()
         assert "name" not in u1.__dict__
@@ -382,13 +405,17 @@ class TestORMInspection(_fixtures.FixtureTest):
         assert "name" in u1.__dict__
 
     def test_attrs_props_prop_added_after_configure(self):
-        class AnonClass(object):
+        class Thing(InspectionAttr):
             pass
 
-        from sqlalchemy.orm import mapper, column_property
+        class AnonClass(object):
+            __foo__ = "bar"
+            __bat__ = Thing()
+
+        from sqlalchemy.orm import column_property
         from sqlalchemy.ext.hybrid import hybrid_property
 
-        m = mapper(AnonClass, self.tables.users)
+        m = self.mapper_registry.map_imperatively(AnonClass, self.tables.users)
 
         eq_(set(inspect(AnonClass).attrs.keys()), set(["id", "name"]))
         eq_(
@@ -409,6 +436,140 @@ class TestORMInspection(_fixtures.FixtureTest):
             set(["id", "name", "q", "foob"]),
         )
 
+    def _random_names(self):
+        import random
+        import keyword
+
+        names = {
+            "".join(
+                random.choice("abcdegfghijklmnopqrstuvwxyz")
+                for i in range(random.randint(3, 15))
+            )
+            for j in range(random.randint(4, 12))
+        }
+        return list(names.difference(keyword.kwlist))
+
+    def _ordered_name_fixture(self, glbls, clsname, base, supercls):
+        import random
+        from sqlalchemy import Integer, Column
+        import textwrap
+
+        names = self._random_names()
+
+        if base is supercls:
+            pk_names = set(
+                random.choice(names) for i in range(random.randint(1, 3))
+            )
+            fk_name = random.choice(
+                [name for name in names if name not in pk_names]
+            )
+        else:
+            pk_names = []
+            fk_name = None
+
+        def _make_name(name):
+            if name in pk_names:
+                return "%s = Column(Integer, primary_key=True)" % name
+            elif name == fk_name:
+                return "%s = Column(ForeignKey('myotherclass.id'))" % name
+            else:
+                type_ = random.choice(["relationship", "column", "hybrid"])
+                if type_ == "relationship":
+                    return "%s = relationship('MyOtherClass')" % name
+                elif type_ == "column":
+                    return "%s = Column(Integer)" % name
+                elif type_ == "hybrid":
+                    return (
+                        "@hybrid_property\ndef %s(self):\n    return None"
+                        % name
+                    )
+
+        glbls["Base"] = base
+        glbls["SuperCls"] = supercls
+
+        if base is supercls:
+
+            class MyOtherClass(base):
+                __tablename__ = "myotherclass"
+                id = Column(Integer, primary_key=True)
+
+            glbls["MyOtherClass"] = MyOtherClass
+        code = """
+
+from sqlalchemy import Column, Integer, ForeignKey
+from sqlalchemy.orm import relationship
+from sqlalchemy.ext.hybrid import hybrid_property
+
+class %s(SuperCls):
+    %s
+
+%s
+""" % (
+            clsname,
+            "__tablename__ = 'mytable'" if base is supercls else "",
+            "\n".join(
+                textwrap.indent(_make_name(name), "    ") for name in names
+            ),
+        )
+
+        exec(code, glbls)
+        return names, glbls[clsname]
+
+    @testing.requires.pep520
+    def test_all_orm_descriptors_pep520_noinh(self):
+        from sqlalchemy.orm import declarative_base
+
+        Base = declarative_base()
+
+        glbls = {}
+        names, MyClass = self._ordered_name_fixture(
+            glbls, "MyClass", Base, Base
+        )
+
+        eq_(MyClass.__mapper__.all_orm_descriptors.keys(), names)
+
+    @testing.requires.pep520
+    def test_all_orm_descriptors_pep520_onelevel_inh(self):
+        from sqlalchemy.orm import declarative_base
+
+        Base = declarative_base()
+
+        glbls = {}
+
+        base_names, MyClass = self._ordered_name_fixture(
+            glbls, "MyClass", Base, Base
+        )
+
+        sub_names, SubClass = self._ordered_name_fixture(
+            glbls, "SubClass", Base, MyClass
+        )
+
+        eq_(
+            SubClass.__mapper__.all_orm_descriptors.keys(),
+            sub_names + base_names,
+        )
+
+    @testing.requires.pep520
+    def test_all_orm_descriptors_pep520_classical(self):
+        class MyClass(object):
+            pass
+
+        from sqlalchemy import Table, MetaData, Column, Integer
+
+        names = self._random_names()
+
+        m = MetaData()
+        t = Table(
+            "t",
+            m,
+            Column("id", Integer, primary_key=True),
+            *[Column(name, Integer) for name in names]
+        )
+
+        m = self.mapper_registry.map_imperatively(MyClass, t)
+
+        eq_(m.all_orm_descriptors.keys(), ["id"] + names)
+
     def test_instance_state_ident_transient(self):
         User = self.classes.User
         u1 = User(name="ed")
@@ -423,7 +584,7 @@ class TestORMInspection(_fixtures.FixtureTest):
         s.flush()
         insp = inspect(u1)
         eq_(insp.identity, (u1.id,))
-        is_(s.query(User).get(insp.identity), u1)
+        is_(s.get(User, insp.identity), u1)
 
     def test_is_instance(self):
         User = self.classes.User
@@ -480,7 +641,7 @@ class TestORMInspection(_fixtures.FixtureTest):
         insp = inspect(u1)
 
         is_(insp.session, None)
-        s = Session()
+        s = fixture_session()
         s.add(u1)
         is_(insp.session, s)
 
