@@ -1,16 +1,13 @@
 from sqlalchemy import exc
 from sqlalchemy import select
 from sqlalchemy import testing
-from sqlalchemy.orm import aliased
 from sqlalchemy.orm import loading
-from sqlalchemy.orm import mapper
 from sqlalchemy.orm import relationship
-from sqlalchemy.orm import Session
 from sqlalchemy.testing import mock
 from sqlalchemy.testing.assertions import assert_raises
 from sqlalchemy.testing.assertions import assert_raises_message
 from sqlalchemy.testing.assertions import eq_
-from sqlalchemy.util import KeyedTuple
+from sqlalchemy.testing.fixtures import fixture_session
 from . import _fixtures
 
 # class GetFromIdentityTest(_fixtures.FixtureTest):
@@ -23,7 +20,7 @@ class InstanceProcessorTest(_fixtures.FixtureTest):
         User, Order, Address = self.classes("User", "Order", "Address")
         users, orders, addresses = self.tables("users", "orders", "addresses")
 
-        mapper(
+        self.mapper_registry.map_imperatively(
             User,
             users,
             properties={
@@ -33,14 +30,14 @@ class InstanceProcessorTest(_fixtures.FixtureTest):
                 ),
             },
         )
-        mapper(
+        self.mapper_registry.map_imperatively(
             Order,
             orders,
             properties={"address": relationship(Address, lazy="joined")},
         )
-        mapper(Address, addresses)
+        self.mapper_registry.map_imperatively(Address, addresses)
 
-        s = Session()
+        s = fixture_session()
 
         def go():
             eq_(
@@ -52,7 +49,7 @@ class InstanceProcessorTest(_fixtures.FixtureTest):
                         Order(id=5, address=None),
                     ],
                 ),
-                s.query(User).populate_existing().get(7),
+                s.get(User, 7, populate_existing=True),
             )
 
         self.assert_sql_count(testing.db, go, 1)
@@ -69,24 +66,24 @@ class InstancesTest(_fixtures.FixtureTest):
 
     def test_cursor_close_w_failed_rowproc(self):
         User = self.classes.User
-        s = Session()
+        s = fixture_session()
 
         q = s.query(User)
 
         ctx = q._compile_context()
         cursor = mock.Mock()
-        q._entities = [
+        ctx.compile_state._entities = [
             mock.Mock(row_processor=mock.Mock(side_effect=Exception("boom")))
         ]
-        assert_raises(Exception, list, loading.instances(q, cursor, ctx))
+        assert_raises(Exception, loading.instances, cursor, ctx)
         assert cursor.close.called, "Cursor wasn't closed"
 
     def test_row_proc_not_created(self):
         User = self.classes.User
-        s = Session()
+        s = fixture_session()
 
         q = s.query(User.id, User.name)
-        stmt = select([User.id])
+        stmt = select(User.id)
 
         assert_raises_message(
             exc.NoSuchColumnError,
@@ -107,7 +104,7 @@ class MergeResultTest(_fixtures.FixtureTest):
     def _fixture(self):
         User = self.classes.User
 
-        s = Session()
+        s = fixture_session()
         u1, u2, u3, u4 = (
             User(id=1, name="u1"),
             User(id=2, name="u2"),
@@ -118,64 +115,37 @@ class MergeResultTest(_fixtures.FixtureTest):
         s.close()
         return s, [u1, u2, u3, u4]
 
-    def test_single_entity(self):
-        s, (u1, u2, u3, u4) = self._fixture()
+    def test_single_entity_frozen(self):
+        s = fixture_session()
         User = self.classes.User
 
-        q = s.query(User)
-        collection = [u1, u2, u3, u4]
-        it = loading.merge_result(q, collection)
-        eq_([x.id for x in it], [1, 2, 7, 8])
+        stmt = select(User).where(User.id.in_([7, 8, 9])).order_by(User.id)
+        result = s.execute(stmt)
+        it = loading.merge_frozen_result(s, stmt, result.freeze())
+        eq_([x.id for x in it().scalars()], [7, 8, 9])
 
-    def test_single_column(self):
+    def test_single_column_frozen(self):
         User = self.classes.User
 
-        s = Session()
+        s = fixture_session()
 
-        q = s.query(User.id)
-        collection = [(1,), (2,), (7,), (8,)]
-        it = loading.merge_result(q, collection)
-        eq_(list(it), [(1,), (2,), (7,), (8,)])
+        stmt = select(User.id).where(User.id.in_([7, 8, 9])).order_by(User.id)
+        result = s.execute(stmt)
+        it = loading.merge_frozen_result(s, stmt, result.freeze())
+        eq_([x.id for x in it()], [7, 8, 9])
 
-    def test_entity_col_mix_plain_tuple(self):
-        s, (u1, u2, u3, u4) = self._fixture()
+    def test_entity_col_mix_plain_tuple_frozen(self):
+        s = fixture_session()
         User = self.classes.User
 
-        q = s.query(User, User.id)
-        collection = [(u1, 1), (u2, 2), (u3, 7), (u4, 8)]
-        it = loading.merge_result(q, collection)
-        it = list(it)
-        eq_([(x.id, y) for x, y in it], [(1, 1), (2, 2), (7, 7), (8, 8)])
-        eq_(list(it[0].keys()), ["User", "id"])
-
-    def test_entity_col_mix_keyed_tuple(self):
-        s, (u1, u2, u3, u4) = self._fixture()
-        User = self.classes.User
-
-        q = s.query(User, User.id)
-
-        def kt(*x):
-            return KeyedTuple(x, ["User", "id"])
-
-        collection = [kt(u1, 1), kt(u2, 2), kt(u3, 7), kt(u4, 8)]
-        it = loading.merge_result(q, collection)
-        it = list(it)
-        eq_([(x.id, y) for x, y in it], [(1, 1), (2, 2), (7, 7), (8, 8)])
-        eq_(list(it[0].keys()), ["User", "id"])
-
-    def test_none_entity(self):
-        s, (u1, u2, u3, u4) = self._fixture()
-        User = self.classes.User
-
-        ua = aliased(User)
-        q = s.query(User, ua)
-
-        def kt(*x):
-            return KeyedTuple(x, ["User", "useralias"])
-
-        collection = [kt(u1, u2), kt(u1, None), kt(u2, u3)]
-        it = loading.merge_result(q, collection)
-        eq_(
-            [(x and x.id or None, y and y.id or None) for x, y in it],
-            [(u1.id, u2.id), (u1.id, None), (u2.id, u3.id)],
+        stmt = (
+            select(User, User.id)
+            .where(User.id.in_([7, 8, 9]))
+            .order_by(User.id)
         )
+        result = s.execute(stmt)
+
+        it = loading.merge_frozen_result(s, stmt, result.freeze())
+        it = list(it())
+        eq_([(x.id, y) for x, y in it], [(7, 7), (8, 8), (9, 9)])
+        eq_(list(it[0]._mapping.keys()), ["User", "id"])
