@@ -9,14 +9,10 @@ from sqlalchemy import testing
 from sqlalchemy.orm import aliased
 from sqlalchemy.orm import attributes
 from sqlalchemy.orm import clear_mappers
-from sqlalchemy.orm import create_session
 from sqlalchemy.orm import exc as orm_exc
 from sqlalchemy.orm import instrumentation
 from sqlalchemy.orm import lazyload
-from sqlalchemy.orm import mapper
 from sqlalchemy.orm import relationship
-from sqlalchemy.orm import Session
-from sqlalchemy.orm import sessionmaker
 from sqlalchemy.orm import state as sa_state
 from sqlalchemy.orm import subqueryload
 from sqlalchemy.orm import with_polymorphic
@@ -25,6 +21,7 @@ from sqlalchemy.orm.collections import column_mapped_collection
 from sqlalchemy.testing import assert_raises_message
 from sqlalchemy.testing import eq_
 from sqlalchemy.testing import fixtures
+from sqlalchemy.testing.fixtures import fixture_session
 from sqlalchemy.testing.pickleable import Address
 from sqlalchemy.testing.pickleable import Child1
 from sqlalchemy.testing.pickleable import Child2
@@ -103,18 +100,18 @@ class PickleTest(fixtures.MappedTest):
             self.tables.dingalings,
         )
 
-        mapper(
+        self.mapper_registry.map_imperatively(
             User,
             users,
             properties={"addresses": relationship(Address, backref="user")},
         )
-        mapper(
+        self.mapper_registry.map_imperatively(
             Address,
             addresses,
             properties={"dingaling": relationship(Dingaling)},
         )
-        mapper(Dingaling, dingalings)
-        sess = create_session()
+        self.mapper_registry.map_imperatively(Dingaling, dingalings)
+        sess = fixture_session()
         u1 = User(name="ed")
         u1.addresses.append(Address(email_address="ed@bar.com"))
         sess.add(u1)
@@ -125,14 +122,14 @@ class PickleTest(fixtures.MappedTest):
     def test_transient(self):
         users, addresses = (self.tables.users, self.tables.addresses)
 
-        mapper(
+        self.mapper_registry.map_imperatively(
             User,
             users,
             properties={"addresses": relationship(Address, backref="user")},
         )
-        mapper(Address, addresses)
+        self.mapper_registry.map_imperatively(Address, addresses)
 
-        sess = create_session()
+        sess = fixture_session()
         u1 = User(name="ed")
         u1.addresses.append(Address(email_address="ed@bar.com"))
 
@@ -142,12 +139,12 @@ class PickleTest(fixtures.MappedTest):
 
         sess.expunge_all()
 
-        eq_(u1, sess.query(User).get(u2.id))
+        eq_(u1, sess.get(User, u2.id))
 
     def test_no_mappers(self):
         users = self.tables.users
 
-        mapper(User, users)
+        self.mapper_registry.map_imperatively(User, users)
         u1 = User(name="ed")
         u1_pickled = pickle.dumps(u1, -1)
 
@@ -164,13 +161,13 @@ class PickleTest(fixtures.MappedTest):
     def test_no_instrumentation(self):
         users = self.tables.users
 
-        mapper(User, users)
+        self.mapper_registry.map_imperatively(User, users)
         u1 = User(name="ed")
         u1_pickled = pickle.dumps(u1, -1)
 
         clear_mappers()
 
-        mapper(User, users)
+        self.mapper_registry.map_imperatively(User, users)
 
         u1 = pickle.loads(u1_pickled)
         # this fails unless the InstanceState
@@ -180,7 +177,7 @@ class PickleTest(fixtures.MappedTest):
     def test_class_deferred_cols(self):
         addresses, users = (self.tables.addresses, self.tables.users)
 
-        mapper(
+        self.mapper_registry.map_imperatively(
             User,
             users,
             properties={
@@ -188,52 +185,57 @@ class PickleTest(fixtures.MappedTest):
                 "addresses": relationship(Address, backref="user"),
             },
         )
-        mapper(
+        self.mapper_registry.map_imperatively(
             Address,
             addresses,
             properties={
                 "email_address": sa.orm.deferred(addresses.c.email_address)
             },
         )
-        sess = create_session()
-        u1 = User(name="ed")
-        u1.addresses.append(Address(email_address="ed@bar.com"))
-        sess.add(u1)
-        sess.flush()
-        sess.expunge_all()
-        u1 = sess.query(User).get(u1.id)
-        assert "name" not in u1.__dict__
-        assert "addresses" not in u1.__dict__
+        with fixture_session(expire_on_commit=False) as sess:
+            u1 = User(name="ed")
+            u1.addresses.append(Address(email_address="ed@bar.com"))
+            sess.add(u1)
+            sess.commit()
+
+        with fixture_session() as sess:
+            u1 = sess.get(User, u1.id)
+            assert "name" not in u1.__dict__
+            assert "addresses" not in u1.__dict__
 
         u2 = pickle.loads(pickle.dumps(u1))
-        sess2 = create_session()
-        sess2.add(u2)
-        eq_(u2.name, "ed")
-        eq_(
-            u2,
-            User(name="ed", addresses=[Address(email_address="ed@bar.com")]),
-        )
+        with fixture_session() as sess2:
+            sess2.add(u2)
+            eq_(u2.name, "ed")
+            eq_(
+                u2,
+                User(
+                    name="ed", addresses=[Address(email_address="ed@bar.com")]
+                ),
+            )
 
         u2 = pickle.loads(pickle.dumps(u1))
-        sess2 = create_session()
-        u2 = sess2.merge(u2, load=False)
-        eq_(u2.name, "ed")
-        eq_(
-            u2,
-            User(name="ed", addresses=[Address(email_address="ed@bar.com")]),
-        )
+        with fixture_session() as sess2:
+            u2 = sess2.merge(u2, load=False)
+            eq_(u2.name, "ed")
+            eq_(
+                u2,
+                User(
+                    name="ed", addresses=[Address(email_address="ed@bar.com")]
+                ),
+            )
 
     def test_instance_lazy_relation_loaders(self):
         users, addresses = (self.tables.users, self.tables.addresses)
 
-        mapper(
+        self.mapper_registry.map_imperatively(
             User,
             users,
             properties={"addresses": relationship(Address, lazy="noload")},
         )
-        mapper(Address, addresses)
+        self.mapper_registry.map_imperatively(Address, addresses)
 
-        sess = Session()
+        sess = fixture_session()
         u1 = User(name="ed", addresses=[Address(email_address="ed@bar.com")])
 
         sess.add(u1)
@@ -243,19 +245,62 @@ class PickleTest(fixtures.MappedTest):
         u1 = sess.query(User).options(lazyload(User.addresses)).first()
         u2 = pickle.loads(pickle.dumps(u1))
 
-        sess = Session()
+        sess = fixture_session()
         sess.add(u2)
         assert u2.addresses
 
+    def test_lazyload_extra_criteria_not_supported(self):
+        users, addresses = (self.tables.users, self.tables.addresses)
+
+        self.mapper_registry.map_imperatively(
+            User,
+            users,
+            properties={"addresses": relationship(Address)},
+        )
+        self.mapper_registry.map_imperatively(Address, addresses)
+
+        sess = fixture_session()
+        u1 = User(
+            name="ed",
+            addresses=[
+                Address(email_address="ed@bar.com"),
+                Address(email_address="ed@wood.com"),
+            ],
+        )
+
+        sess.add(u1)
+        sess.commit()
+        sess.close()
+
+        u1 = (
+            sess.query(User)
+            .options(
+                lazyload(
+                    User.addresses.and_(Address.email_address == "ed@bar.com")
+                )
+            )
+            .first()
+        )
+        with testing.expect_warnings(
+            r"Can't reliably serialize a lazyload\(\) option"
+        ):
+            u2 = pickle.loads(pickle.dumps(u1))
+
+        eq_(len(u1.addresses), 1)
+
+        sess = fixture_session()
+        sess.add(u2)
+        eq_(len(u2.addresses), 2)
+
     def test_invalidated_flag_pickle(self):
         users, addresses = (self.tables.users, self.tables.addresses)
 
-        mapper(
+        self.mapper_registry.map_imperatively(
             User,
             users,
             properties={"addresses": relationship(Address, lazy="noload")},
         )
-        mapper(Address, addresses)
+        self.mapper_registry.map_imperatively(Address, addresses)
 
         u1 = User()
         u1.addresses.append(Address())
@@ -266,12 +311,12 @@ class PickleTest(fixtures.MappedTest):
     def test_invalidated_flag_deepcopy(self):
         users, addresses = (self.tables.users, self.tables.addresses)
 
-        mapper(
+        self.mapper_registry.map_imperatively(
             User,
             users,
             properties={"addresses": relationship(Address, lazy="noload")},
         )
-        mapper(Address, addresses)
+        self.mapper_registry.map_imperatively(Address, addresses)
 
         u1 = User()
         u1.addresses.append(Address())
@@ -283,71 +328,78 @@ class PickleTest(fixtures.MappedTest):
     def test_instance_deferred_cols(self):
         users, addresses = (self.tables.users, self.tables.addresses)
 
-        mapper(
+        self.mapper_registry.map_imperatively(
             User,
             users,
             properties={"addresses": relationship(Address, backref="user")},
         )
-        mapper(Address, addresses)
+        self.mapper_registry.map_imperatively(Address, addresses)
 
-        sess = create_session()
-        u1 = User(name="ed")
-        u1.addresses.append(Address(email_address="ed@bar.com"))
-        sess.add(u1)
-        sess.flush()
-        sess.expunge_all()
+        with fixture_session(expire_on_commit=False) as sess:
+            u1 = User(name="ed")
+            u1.addresses.append(Address(email_address="ed@bar.com"))
+            sess.add(u1)
+            sess.commit()
 
-        u1 = (
-            sess.query(User)
-            .options(
-                sa.orm.defer("name"), sa.orm.defer("addresses.email_address")
+        with fixture_session(expire_on_commit=False) as sess:
+            u1 = sess.get(
+                User,
+                u1.id,
+                options=[
+                    sa.orm.defer(User.name),
+                    sa.orm.defaultload(User.addresses).defer(
+                        Address.email_address
+                    ),
+                ],
             )
-            .get(u1.id)
-        )
-        assert "name" not in u1.__dict__
-        assert "addresses" not in u1.__dict__
+            assert "name" not in u1.__dict__
+            assert "addresses" not in u1.__dict__
 
         u2 = pickle.loads(pickle.dumps(u1))
-        sess2 = create_session()
-        sess2.add(u2)
-        eq_(u2.name, "ed")
-        assert "addresses" not in u2.__dict__
-        ad = u2.addresses[0]
-        assert "email_address" not in ad.__dict__
-        eq_(ad.email_address, "ed@bar.com")
-        eq_(
-            u2,
-            User(name="ed", addresses=[Address(email_address="ed@bar.com")]),
-        )
+        with fixture_session() as sess2:
+            sess2.add(u2)
+            eq_(u2.name, "ed")
+            assert "addresses" not in u2.__dict__
+            ad = u2.addresses[0]
+            assert "email_address" not in ad.__dict__
+            eq_(ad.email_address, "ed@bar.com")
+            eq_(
+                u2,
+                User(
+                    name="ed", addresses=[Address(email_address="ed@bar.com")]
+                ),
+            )
 
         u2 = pickle.loads(pickle.dumps(u1))
-        sess2 = create_session()
-        u2 = sess2.merge(u2, load=False)
-        eq_(u2.name, "ed")
-        assert "addresses" not in u2.__dict__
-        ad = u2.addresses[0]
+        with fixture_session() as sess2:
+            u2 = sess2.merge(u2, load=False)
+            eq_(u2.name, "ed")
+            assert "addresses" not in u2.__dict__
+            ad = u2.addresses[0]
 
-        # mapper options now transmit over merge(),
-        # new as of 0.6, so email_address is deferred.
-        assert "email_address" not in ad.__dict__
+            # mapper options now transmit over merge(),
+            # new as of 0.6, so email_address is deferred.
+            assert "email_address" not in ad.__dict__
 
-        eq_(ad.email_address, "ed@bar.com")
-        eq_(
-            u2,
-            User(name="ed", addresses=[Address(email_address="ed@bar.com")]),
-        )
+            eq_(ad.email_address, "ed@bar.com")
+            eq_(
+                u2,
+                User(
+                    name="ed", addresses=[Address(email_address="ed@bar.com")]
+                ),
+            )
 
     def test_pickle_protocols(self):
         users, addresses = (self.tables.users, self.tables.addresses)
 
-        mapper(
+        self.mapper_registry.map_imperatively(
             User,
             users,
             properties={"addresses": relationship(Address, backref="user")},
         )
-        mapper(Address, addresses)
+        self.mapper_registry.map_imperatively(Address, addresses)
 
-        sess = sessionmaker()()
+        sess = fixture_session()
         u1 = User(name="ed")
         u1.addresses.append(Address(email_address="ed@bar.com"))
         sess.add(u1)
@@ -362,8 +414,8 @@ class PickleTest(fixtures.MappedTest):
 
     def test_09_pickle(self):
         users = self.tables.users
-        mapper(User, users)
-        sess = Session()
+        self.mapper_registry.map_imperatively(User, users)
+        sess = fixture_session()
         sess.add(User(id=1, name="ed"))
         sess.commit()
         sess.close()
@@ -389,7 +441,7 @@ class PickleTest(fixtures.MappedTest):
         state.__setstate__(state_09)
         eq_(state.expired_attributes, {"name", "id"})
 
-        sess = Session()
+        sess = fixture_session()
         sess.add(inst)
         eq_(inst.name, "ed")
         # test identity_token expansion
@@ -397,8 +449,8 @@ class PickleTest(fixtures.MappedTest):
 
     def test_11_pickle(self):
         users = self.tables.users
-        mapper(User, users)
-        sess = Session()
+        self.mapper_registry.map_imperatively(User, users)
+        sess = fixture_session()
         u1 = User(id=1, name="ed")
         sess.add(u1)
         sess.commit()
@@ -429,7 +481,7 @@ class PickleTest(fixtures.MappedTest):
 
     def test_state_info_pickle(self):
         users = self.tables.users
-        mapper(User, users)
+        self.mapper_registry.map_imperatively(User, users)
 
         u1 = User(id=1, name="ed")
 
@@ -444,42 +496,48 @@ class PickleTest(fixtures.MappedTest):
         eq_(sa.inspect(u2).info["some_key"], "value")
 
     @testing.requires.non_broken_pickle
-    def test_unbound_options(self):
+    @testing.combinations(
+        lambda User: sa.orm.joinedload(User.addresses),
+        lambda User: sa.orm.defer(User.name),
+        lambda Address: sa.orm.joinedload(User.addresses).joinedload(
+            Address.dingaling
+        ),
+        lambda: sa.orm.joinedload(User.addresses).raiseload("*"),
+        lambda: sa.orm.raiseload("*"),
+    )
+    def test_unbound_options(self, test_case):
         sess, User, Address, Dingaling = self._option_test_fixture()
 
-        for opt in [
-            sa.orm.joinedload(User.addresses),
-            sa.orm.joinedload("addresses"),
-            sa.orm.defer("name"),
-            sa.orm.defer(User.name),
-            sa.orm.joinedload("addresses").joinedload(Address.dingaling),
-        ]:
-            opt2 = pickle.loads(pickle.dumps(opt))
-            eq_(opt.path, opt2.path)
+        opt = testing.resolve_lambda(test_case, User=User, Address=Address)
+        opt2 = pickle.loads(pickle.dumps(opt))
+        eq_(opt.path, opt2.path)
 
         u1 = sess.query(User).options(opt).first()
         pickle.loads(pickle.dumps(u1))
 
     @testing.requires.non_broken_pickle
-    def test_bound_options(self):
+    @testing.combinations(
+        lambda User: sa.orm.Load(User).joinedload(User.addresses),
+        lambda User: sa.orm.Load(User)
+        .joinedload(User.addresses)
+        .raiseload("*"),
+        lambda User: sa.orm.Load(User).defer(User.name),
+        lambda User, Address: sa.orm.Load(User)
+        .joinedload(User.addresses)
+        .joinedload(Address.dingaling),
+        lambda User, Address: sa.orm.Load(User)
+        .joinedload(User.addresses, innerjoin=True)
+        .joinedload(Address.dingaling),
+    )
+    def test_bound_options(self, test_case):
         sess, User, Address, Dingaling = self._option_test_fixture()
 
-        for opt in [
-            sa.orm.Load(User).joinedload(User.addresses),
-            sa.orm.Load(User).joinedload("addresses"),
-            sa.orm.Load(User).defer("name"),
-            sa.orm.Load(User).defer(User.name),
-            sa.orm.Load(User)
-            .joinedload("addresses")
-            .joinedload(Address.dingaling),
-            sa.orm.Load(User)
-            .joinedload("addresses", innerjoin=True)
-            .joinedload(Address.dingaling),
-        ]:
-            opt2 = pickle.loads(pickle.dumps(opt))
-            eq_(opt.path, opt2.path)
-            eq_(opt.context.keys(), opt2.context.keys())
-            eq_(opt.local_opts, opt2.local_opts)
+        opt = testing.resolve_lambda(test_case, User=User, Address=Address)
+
+        opt2 = pickle.loads(pickle.dumps(opt))
+        eq_(opt.path, opt2.path)
+        eq_(opt.context.keys(), opt2.context.keys())
+        eq_(opt.local_opts, opt2.local_opts)
 
         u1 = sess.query(User).options(opt).first()
         pickle.loads(pickle.dumps(u1))
@@ -490,14 +548,14 @@ class PickleTest(fixtures.MappedTest):
 
         for opt in [
             sa.orm.joinedload(User.addresses),
-            sa.orm.joinedload("addresses"),
-            sa.orm.defer("name"),
             sa.orm.defer(User.name),
-            sa.orm.joinedload("addresses").joinedload(Address.dingaling),
+            sa.orm.joinedload(User.addresses).joinedload(Address.dingaling),
         ]:
-            q = sess.query(User).options(opt)
+            context = sess.query(User).options(opt)._compile_context()
             opt = [
-                v for v in q._attributes.values() if isinstance(v, sa.orm.Load)
+                v
+                for v in context.attributes.values()
+                if isinstance(v, sa.orm.Load)
             ][0]
 
             opt2 = pickle.loads(pickle.dumps(opt))
@@ -524,7 +582,7 @@ class PickleTest(fixtures.MappedTest):
         )
         p = Table("p", m, Column("id", String, primary_key=True))
 
-        mapper(
+        self.mapper_registry.map_imperatively(
             Parent,
             p,
             properties={
@@ -532,8 +590,8 @@ class PickleTest(fixtures.MappedTest):
                 "children2": relationship(Child2),
             },
         )
-        mapper(Child1, c1)
-        mapper(Child2, c2)
+        self.mapper_registry.map_imperatively(Child1, c1)
+        self.mapper_registry.map_imperatively(Child2, c2)
 
         obj = Parent()
         screen1 = Screen(obj)
@@ -546,7 +604,7 @@ class PickleTest(fixtures.MappedTest):
             pass
 
         users = self.tables.users
-        mapper(User, users)
+        self.mapper_registry.map_imperatively(User, users)
 
         for sa_exc in (
             orm_exc.UnmappedInstanceError(Foo()),
@@ -560,7 +618,7 @@ class PickleTest(fixtures.MappedTest):
     def test_attribute_mapped_collection(self):
         users, addresses = self.tables.users, self.tables.addresses
 
-        mapper(
+        self.mapper_registry.map_imperatively(
             User,
             users,
             properties={
@@ -572,7 +630,7 @@ class PickleTest(fixtures.MappedTest):
                 )
             },
         )
-        mapper(Address, addresses)
+        self.mapper_registry.map_imperatively(Address, addresses)
         u1 = User()
         u1.addresses = {"email1": Address(email_address="email1")}
         for loads, dumps in picklers():
@@ -583,7 +641,7 @@ class PickleTest(fixtures.MappedTest):
     def test_column_mapped_collection(self):
         users, addresses = self.tables.users, self.tables.addresses
 
-        mapper(
+        self.mapper_registry.map_imperatively(
             User,
             users,
             properties={
@@ -595,7 +653,7 @@ class PickleTest(fixtures.MappedTest):
                 )
             },
         )
-        mapper(Address, addresses)
+        self.mapper_registry.map_imperatively(Address, addresses)
         u1 = User()
         u1.addresses = {
             "email1": Address(email_address="email1"),
@@ -609,7 +667,7 @@ class PickleTest(fixtures.MappedTest):
     def test_composite_column_mapped_collection(self):
         users, addresses = self.tables.users, self.tables.addresses
 
-        mapper(
+        self.mapper_registry.map_imperatively(
             User,
             users,
             properties={
@@ -621,7 +679,7 @@ class PickleTest(fixtures.MappedTest):
                 )
             },
         )
-        mapper(Address, addresses)
+        self.mapper_registry.map_imperatively(Address, addresses)
         u1 = User()
         u1.addresses = {
             (1, "email1"): Address(id=1, email_address="email1"),
@@ -656,7 +714,7 @@ class OptionsTest(_Polymorphic):
             eq_(opt2.__getstate__()["path"], serialized)
 
     def test_load(self):
-        s = Session()
+        s = fixture_session()
 
         with_poly = with_polymorphic(Person, [Engineer, Manager], flat=True)
         emp = (
@@ -690,13 +748,13 @@ class PolymorphicDeferredTest(fixtures.MappedTest):
     def test_polymorphic_deferred(self):
         email_users, users = (self.tables.email_users, self.tables.users)
 
-        mapper(
+        self.mapper_registry.map_imperatively(
             User,
             users,
             polymorphic_identity="user",
             polymorphic_on=users.c.type,
         )
-        mapper(
+        self.mapper_registry.map_imperatively(
             EmailUser,
             email_users,
             inherits=User,
@@ -704,17 +762,17 @@ class PolymorphicDeferredTest(fixtures.MappedTest):
         )
 
         eu = EmailUser(name="user1", email_address="foo@bar.com")
-        sess = create_session()
-        sess.add(eu)
-        sess.flush()
-        sess.expunge_all()
+        with fixture_session() as sess:
+            sess.add(eu)
+            sess.commit()
 
-        eu = sess.query(User).first()
-        eu2 = pickle.loads(pickle.dumps(eu))
-        sess2 = create_session()
-        sess2.add(eu2)
-        assert "email_address" not in eu2.__dict__
-        eq_(eu2.email_address, "foo@bar.com")
+        with fixture_session() as sess:
+            eu = sess.query(User).first()
+            eu2 = pickle.loads(pickle.dumps(eu))
+            sess2 = fixture_session()
+            sess2.add(eu2)
+            assert "email_address" not in eu2.__dict__
+            eq_(eu2.email_address, "foo@bar.com")
 
 
 class TupleLabelTest(_fixtures.FixtureTest):
@@ -729,7 +787,7 @@ class TupleLabelTest(_fixtures.FixtureTest):
             cls.tables.addresses,
             cls.tables.orders,
         )
-        mapper(
+        cls.mapper_registry.map_imperatively(
             User,
             users,
             properties={
@@ -742,13 +800,13 @@ class TupleLabelTest(_fixtures.FixtureTest):
                 ),
             },
         )
-        mapper(Address, addresses)
-        mapper(
+        cls.mapper_registry.map_imperatively(Address, addresses)
+        cls.mapper_registry.map_imperatively(
             Order, orders, properties={"address": relationship(Address)}
         )  # m2o
 
     def test_tuple_labeling(self):
-        sess = create_session()
+        sess = fixture_session()
 
         # test pickle + all the protocols !
         for pickled in False, -1, 0, 1, 2:
@@ -756,31 +814,35 @@ class TupleLabelTest(_fixtures.FixtureTest):
                 if pickled is not False:
                     row = pickle.loads(pickle.dumps(row, pickled))
 
-                eq_(list(row.keys()), ["User", "Address"])
+                eq_(list(row._fields), ["User", "Address"])
                 eq_(row.User, row[0])
                 eq_(row.Address, row[1])
 
             for row in sess.query(User.name, User.id.label("foobar")):
                 if pickled is not False:
                     row = pickle.loads(pickle.dumps(row, pickled))
-                eq_(list(row.keys()), ["name", "foobar"])
+                eq_(list(row._fields), ["name", "foobar"])
                 eq_(row.name, row[0])
                 eq_(row.foobar, row[1])
 
-            for row in sess.query(User).values(
+            for row in sess.query(User).with_entities(
                 User.name, User.id.label("foobar")
             ):
                 if pickled is not False:
                     row = pickle.loads(pickle.dumps(row, pickled))
-                eq_(list(row.keys()), ["name", "foobar"])
+                eq_(list(row._fields), ["name", "foobar"])
                 eq_(row.name, row[0])
                 eq_(row.foobar, row[1])
 
             oalias = aliased(Order)
-            for row in sess.query(User, oalias).join(User.orders).all():
+            for row in (
+                sess.query(User, oalias)
+                .join(User.orders.of_type(oalias))
+                .all()
+            ):
                 if pickled is not False:
                     row = pickle.loads(pickle.dumps(row, pickled))
-                eq_(list(row.keys()), ["User"])
+                eq_(list(row._fields), ["User"])
                 eq_(row.User, row[0])
 
             oalias = aliased(Order, name="orders")
@@ -789,14 +851,12 @@ class TupleLabelTest(_fixtures.FixtureTest):
             ):
                 if pickled is not False:
                     row = pickle.loads(pickle.dumps(row, pickled))
-                eq_(list(row.keys()), ["User", "orders"])
+                eq_(list(row._fields), ["User", "orders"])
                 eq_(row.User, row[0])
                 eq_(row.orders, row[1])
 
-            # test here that first col is not labeled, only
-            # one name in keys, matches correctly
             for row in sess.query(User.name + "hoho", User.name):
-                eq_(list(row.keys()), ["name"])
+                eq_(list(row._fields), ["name"])
                 eq_(row[0], row.name + "hoho")
 
             if pickled is not False:
@@ -838,7 +898,7 @@ class CustomSetupTeardownTest(fixtures.MappedTest):
 
         users = self.tables.users
 
-        mapper(User, users)
+        self.mapper_registry.map_imperatively(User, users)
 
         u1 = User()
         attributes.manager_of_class(User).teardown_instance(u1)
