@@ -26,7 +26,7 @@ from sqlalchemy.testing import mock
 
 class _JoinFixtures(object):
     @classmethod
-    def setup_class(cls):
+    def setup_test_class(cls):
         m = MetaData()
         cls.left = Table(
             "lft",
@@ -470,7 +470,7 @@ class _JoinFixtures(object):
             self.left,
             self.right,
             primaryjoin=self.left.c.id == func.foo(self.right.c.lid),
-            consider_as_foreign_keys=[self.right.c.lid],
+            consider_as_foreign_keys={self.right.c.lid},
             **kw
         )
 
@@ -480,10 +480,10 @@ class _JoinFixtures(object):
             self.composite_multi_ref,
             self.composite_target,
             self.composite_multi_ref,
-            consider_as_foreign_keys=[
+            consider_as_foreign_keys={
                 self.composite_multi_ref.c.uid2,
                 self.composite_multi_ref.c.oid,
-            ],
+            },
             **kw
         )
 
@@ -807,7 +807,7 @@ class ColumnCollectionsTest(
         )
 
     def test_determine_local_remote_pairs_o2m_composite_selfref_func_warning(
-        self
+        self,
     ):
         self._assert_non_simple_warning(
             self._join_fixture_o2m_composite_selfref_func
@@ -823,7 +823,7 @@ class ColumnCollectionsTest(
         )
 
     def test_determine_local_remote_pairs_o2m_composite_selfref_func_annotated(
-        self
+        self,
     ):
         joincond = self._join_fixture_o2m_composite_selfref_func_annotated()
         eq_(
@@ -1099,10 +1099,10 @@ class DetermineJoinTest(_JoinFixtures, fixtures.TestBase, AssertsCompiledSQL):
             self.m2mleft,
             self.m2mright,
             secondary=self.m2msecondary_ambig_fks,
-            consider_as_foreign_keys=[
+            consider_as_foreign_keys={
                 self.m2msecondary_ambig_fks.c.lid1,
                 self.m2msecondary_ambig_fks.c.rid1,
-            ],
+            },
         )
 
     def test_determine_join_w_fks_ambig_m2m(self):
@@ -1120,46 +1120,53 @@ class AdaptedJoinTest(_JoinFixtures, fixtures.TestBase, AssertsCompiledSQL):
 
     def test_join_targets_o2m_selfref(self):
         joincond = self._join_fixture_o2m_selfref()
-        left = select([joincond.parent_selectable]).alias("pj")
+        left = select(joincond.parent_persist_selectable).alias("pj")
         pj, sj, sec, adapter, ds = joincond.join_targets(
-            left, joincond.child_selectable, True
+            left, joincond.child_persist_selectable, True
         )
         self.assert_compile(pj, "pj.id = selfref.sid")
+        self.assert_compile(pj, "pj.id = selfref.sid")
 
-        right = select([joincond.child_selectable]).alias("pj")
+        right = select(joincond.child_persist_selectable).alias("pj")
         pj, sj, sec, adapter, ds = joincond.join_targets(
-            joincond.parent_selectable, right, True
+            joincond.parent_persist_selectable, right, True
         )
         self.assert_compile(pj, "selfref.id = pj.sid")
+        self.assert_compile(pj, "selfref.id = pj.sid")
 
     def test_join_targets_o2m_plain(self):
         joincond = self._join_fixture_o2m()
         pj, sj, sec, adapter, ds = joincond.join_targets(
-            joincond.parent_selectable, joincond.child_selectable, False
+            joincond.parent_persist_selectable,
+            joincond.child_persist_selectable,
+            False,
         )
         self.assert_compile(pj, "lft.id = rgt.lid")
+        self.assert_compile(pj, "lft.id = rgt.lid")
 
     def test_join_targets_o2m_left_aliased(self):
         joincond = self._join_fixture_o2m()
-        left = select([joincond.parent_selectable]).alias("pj")
+        left = select(joincond.parent_persist_selectable).alias("pj")
         pj, sj, sec, adapter, ds = joincond.join_targets(
-            left, joincond.child_selectable, True
+            left, joincond.child_persist_selectable, True
         )
         self.assert_compile(pj, "pj.id = rgt.lid")
+        self.assert_compile(pj, "pj.id = rgt.lid")
 
     def test_join_targets_o2m_right_aliased(self):
         joincond = self._join_fixture_o2m()
-        right = select([joincond.child_selectable]).alias("pj")
+        right = select(joincond.child_persist_selectable).alias("pj")
         pj, sj, sec, adapter, ds = joincond.join_targets(
-            joincond.parent_selectable, right, True
+            joincond.parent_persist_selectable, right, True
         )
         self.assert_compile(pj, "lft.id = pj.lid")
+        self.assert_compile(pj, "lft.id = pj.lid")
 
     def test_join_targets_o2m_composite_selfref(self):
         joincond = self._join_fixture_o2m_composite_selfref()
-        right = select([joincond.child_selectable]).alias("pj")
+        right = select(joincond.child_persist_selectable).alias("pj")
         pj, sj, sec, adapter, ds = joincond.join_targets(
-            joincond.parent_selectable, right, True
+            joincond.parent_persist_selectable, right, True
         )
         self.assert_compile(
             pj,
@@ -1169,9 +1176,9 @@ class AdaptedJoinTest(_JoinFixtures, fixtures.TestBase, AssertsCompiledSQL):
 
     def test_join_targets_m2o_composite_selfref(self):
         joincond = self._join_fixture_m2o_composite_selfref()
-        right = select([joincond.child_selectable]).alias("pj")
+        right = select(joincond.child_persist_selectable).alias("pj")
         pj, sj, sec, adapter, ds = joincond.join_targets(
-            joincond.parent_selectable, right, True
+            joincond.parent_persist_selectable, right, True
         )
         self.assert_compile(
             pj,
@@ -1232,7 +1239,7 @@ class LazyClauseTest(_JoinFixtures, fixtures.TestBase, AssertsCompiledSQL):
 
 class DeannotateCorrectlyTest(fixtures.TestBase):
     def test_pj_deannotates(self):
-        from sqlalchemy.ext.declarative import declarative_base
+        from sqlalchemy.orm import declarative_base
 
         Base = declarative_base()
 
