@@ -13,9 +13,11 @@ from sqlalchemy import Sequence
 from sqlalchemy import String
 from sqlalchemy import Table
 from sqlalchemy import table
+from sqlalchemy import testing
 from sqlalchemy import text
 from sqlalchemy.dialects import mysql
 from sqlalchemy.dialects import postgresql
+from sqlalchemy.dialects import sqlite
 from sqlalchemy.engine import default
 from sqlalchemy.sql import crud
 from sqlalchemy.testing import assert_raises
@@ -26,6 +28,14 @@ from sqlalchemy.testing import expect_warnings
 from sqlalchemy.testing import fixtures
 
 
+class ORMExpr(object):
+    def __init__(self, col):
+        self.col = col
+
+    def __clause_element__(self):
+        return self.col
+
+
 class _InsertTestBase(object):
     @classmethod
     def define_tables(cls, metadata):
@@ -129,12 +139,19 @@ class InsertTest(_InsertTestBase, fixtures.TablesTest, AssertsCompiledSQL):
             Column(
                 "col2",
                 Integer,
-                default=select([func.coalesce(func.max(foo.c.id))]),
+                default=select(func.coalesce(func.max(foo.c.id))),
             ),
         )
 
         self.assert_compile(
-            t.insert(inline=True, values={}),
+            t.insert().values({}),
+            "INSERT INTO test (col1, col2) VALUES (foo(:foo_1), "
+            "(SELECT coalesce(max(foo.id)) AS coalesce_1 FROM "
+            "foo))",
+        )
+
+        self.assert_compile(
+            t.insert().inline().values({}),
             "INSERT INTO test (col1, col2) VALUES (foo(:foo_1), "
             "(SELECT coalesce(max(foo.id)) AS coalesce_1 FROM "
             "foo))",
@@ -155,7 +172,7 @@ class InsertTest(_InsertTestBase, fixtures.TablesTest, AssertsCompiledSQL):
         checkparams = {"myid": 3, "name": "jack"}
 
         self.assert_compile(
-            insert(table1, dict(myid=3, name="jack")),
+            insert(table1).values(myid=3, name="jack"),
             "INSERT INTO mytable (myid, name) VALUES (:myid, :name)",
             checkparams=checkparams,
         )
@@ -185,7 +202,7 @@ class InsertTest(_InsertTestBase, fixtures.TablesTest, AssertsCompiledSQL):
 
         checkparams = {"myid": 3, "name": "jack", "unknowncol": "oops"}
 
-        stmt = insert(table1, values=checkparams)
+        stmt = insert(table1).values(checkparams)
         assert_raises_message(
             exc.CompileError,
             "Unconsumed column names: unknowncol",
@@ -201,7 +218,7 @@ class InsertTest(_InsertTestBase, fixtures.TablesTest, AssertsCompiledSQL):
             {"myid": 4, "name": "someone", "unknowncol": "oops"},
         ]
 
-        stmt = insert(table1, values=checkparams)
+        stmt = insert(table1).values(checkparams)
         assert_raises_message(
             exc.CompileError,
             "Unconsumed column names: unknowncol",
@@ -219,7 +236,7 @@ class InsertTest(_InsertTestBase, fixtures.TablesTest, AssertsCompiledSQL):
         }
 
         self.assert_compile(
-            insert(table1, (3, "jack", "mydescription")),
+            insert(table1).values([3, "jack", "mydescription"]),
             "INSERT INTO mytable (myid, name, description) "
             "VALUES (:myid, :name, :description)",
             checkparams=checkparams,
@@ -229,7 +246,7 @@ class InsertTest(_InsertTestBase, fixtures.TablesTest, AssertsCompiledSQL):
         table1 = self.tables.mytable
 
         self.assert_compile(
-            insert(table1, values=dict(myid=func.lala())),
+            insert(table1).values(myid=func.lala()),
             "INSERT INTO mytable (myid) VALUES (lala())",
         )
 
@@ -242,18 +259,18 @@ class InsertTest(_InsertTestBase, fixtures.TablesTest, AssertsCompiledSQL):
         }
 
         self.assert_compile(
-            insert(table1, values),
+            insert(table1).values(values),
             "INSERT INTO mytable (myid, name) VALUES (:userid, :username)",
         )
 
-    def test_insert_values(self):
+    def test_insert_values_multiple(self):
         table1 = self.tables.mytable
 
         values1 = {table1.c.myid: bindparam("userid")}
         values2 = {table1.c.name: bindparam("username")}
 
         self.assert_compile(
-            insert(table1, values=values1).values(values2),
+            insert(table1).values(values1).values(values2),
             "INSERT INTO mytable (myid, name) VALUES (:userid, :username)",
         )
 
@@ -289,12 +306,23 @@ class InsertTest(_InsertTestBase, fixtures.TablesTest, AssertsCompiledSQL):
         )
 
         self.assert_compile(
-            table.insert(values={}, inline=True),
+            table.insert().values(),
             "INSERT INTO sometable (foo) VALUES (foobar())",
         )
 
         self.assert_compile(
-            table.insert(inline=True),
+            table.insert(),
+            "INSERT INTO sometable (foo) VALUES (foobar())",
+            params={},
+        )
+
+        self.assert_compile(
+            table.insert().values().inline(),
+            "INSERT INTO sometable (foo) VALUES (foobar())",
+        )
+
+        self.assert_compile(
+            table.insert().inline(),
             "INSERT INTO sometable (foo) VALUES (foobar())",
             params={},
         )
@@ -312,7 +340,7 @@ class InsertTest(_InsertTestBase, fixtures.TablesTest, AssertsCompiledSQL):
 
     def test_insert_from_select_returning(self):
         table1 = self.tables.mytable
-        sel = select([table1.c.myid, table1.c.name]).where(
+        sel = select(table1.c.myid, table1.c.name).where(
             table1.c.name == "foo"
         )
         ins = (
@@ -331,7 +359,7 @@ class InsertTest(_InsertTestBase, fixtures.TablesTest, AssertsCompiledSQL):
 
     def test_insert_from_select_select(self):
         table1 = self.tables.mytable
-        sel = select([table1.c.myid, table1.c.name]).where(
+        sel = select(table1.c.myid, table1.c.name).where(
             table1.c.name == "foo"
         )
         ins = self.tables.myothertable.insert().from_select(
@@ -355,7 +383,7 @@ class InsertTest(_InsertTestBase, fixtures.TablesTest, AssertsCompiledSQL):
             Column("data", String),
         )
 
-        stmt = t1.insert().from_select(("data",), select([t1.c.data]))
+        stmt = t1.insert().from_select(("data",), select(t1.c.data))
 
         self.assert_compile(
             stmt,
@@ -364,12 +392,81 @@ class InsertTest(_InsertTestBase, fixtures.TablesTest, AssertsCompiledSQL):
             dialect=postgresql.dialect(),
         )
 
+    def test_insert_seq_pk_multi_values(self):
+        """test #6361"""
+
+        m = MetaData()
+
+        t1 = Table(
+            "t",
+            m,
+            Column("id", Integer, Sequence("id_seq"), primary_key=True),
+            Column("data", String),
+        )
+
+        stmt = t1.insert().values(
+            [{"data": "d1"}, {"data": "d2"}, {"data": "d3"}]
+        )
+
+        self.assert_compile(
+            stmt,
+            "INSERT INTO t (id, data) VALUES (nextval('id_seq'), "
+            "%(data_m0)s), (nextval('id_seq'), %(data_m1)s), "
+            "(nextval('id_seq'), %(data_m2)s)",
+            dialect=postgresql.dialect(),
+        )
+
+    def test_insert_seq_non_pk_multi_values(self):
+        """test #6361"""
+
+        m = MetaData()
+
+        t1 = Table(
+            "t",
+            m,
+            Column("id", Integer, primary_key=True),
+            Column("counter", Sequence("counter_seq")),
+            Column("data", String),
+        )
+
+        stmt = t1.insert().values(
+            [{"data": "d1"}, {"data": "d2"}, {"data": "d3"}]
+        )
+
+        self.assert_compile(
+            stmt,
+            "INSERT INTO t (counter, data) VALUES (nextval('counter_seq'), "
+            "%(data_m0)s), (nextval('counter_seq'), %(data_m1)s), "
+            "(nextval('counter_seq'), %(data_m2)s)",
+            dialect=postgresql.dialect(),
+        )
+
+    def test_insert_seq_pk_multi_values_seq_not_supported(self):
+        m = MetaData()
+
+        t1 = Table(
+            "t",
+            m,
+            Column("id", Integer, Sequence("id_seq"), primary_key=True),
+            Column("data", String),
+        )
+
+        stmt = t1.insert().values(
+            [{"data": "d1"}, {"data": "d2"}, {"data": "d3"}]
+        )
+
+        self.assert_compile(
+            stmt,
+            "INSERT INTO t (data) VALUES (?), (?), (?)",
+            dialect=sqlite.dialect(),
+        )
+
     def test_insert_from_select_cte_one(self):
         table1 = self.tables.mytable
 
-        cte = select([table1.c.name]).where(table1.c.name == "bar").cte()
+        cte = select(table1.c.name).where(table1.c.name == "bar").cte()
 
-        sel = select([table1.c.myid, table1.c.name]).where(
+        sel = select(table1.c.myid, table1.c.name).where(
             table1.c.name == cte.c.name
         )
 
@@ -393,9 +490,9 @@ class InsertTest(_InsertTestBase, fixtures.TablesTest, AssertsCompiledSQL):
 
         table1 = self.tables.mytable
 
-        cte = select([table1.c.name]).where(table1.c.name == "bar").cte()
+        cte = select(table1.c.name).where(table1.c.name == "bar").cte()
 
-        sel = select([table1.c.myid, table1.c.name]).where(
+        sel = select(table1.c.myid, table1.c.name).where(
             table1.c.name == cte.c.name
         )
 
@@ -449,7 +546,7 @@ class InsertTest(_InsertTestBase, fixtures.TablesTest, AssertsCompiledSQL):
 
     def test_insert_from_select_select_alt_ordering(self):
         table1 = self.tables.mytable
-        sel = select([table1.c.name, table1.c.myid]).where(
+        sel = select(table1.c.name, table1.c.myid).where(
             table1.c.name == "foo"
         )
         ins = self.tables.myothertable.insert().from_select(
@@ -472,7 +569,7 @@ class InsertTest(_InsertTestBase, fixtures.TablesTest, AssertsCompiledSQL):
             Column("foo", Integer, default=func.foobar()),
         )
         table1 = self.tables.mytable
-        sel = select([table1.c.myid]).where(table1.c.name == "foo")
+        sel = select(table1.c.myid).where(table1.c.name == "foo")
         ins = table.insert().from_select(["id"], sel, include_defaults=False)
         self.assert_compile(
             ins,
@@ -490,7 +587,7 @@ class InsertTest(_InsertTestBase, fixtures.TablesTest, AssertsCompiledSQL):
             Column("foo", Integer, default=func.foobar()),
         )
         table1 = self.tables.mytable
-        sel = select([table1.c.myid]).where(table1.c.name == "foo")
+        sel = select(table1.c.myid).where(table1.c.name == "foo")
         ins = table.insert().from_select(["id"], sel)
         self.assert_compile(
             ins,
@@ -509,7 +606,7 @@ class InsertTest(_InsertTestBase, fixtures.TablesTest, AssertsCompiledSQL):
             Column("foo", Integer, default=12),
         )
         table1 = self.tables.mytable
-        sel = select([table1.c.myid]).where(table1.c.name == "foo")
+        sel = select(table1.c.myid).where(table1.c.name == "foo")
         ins = table.insert().from_select(["id"], sel)
         self.assert_compile(
             ins,
@@ -529,7 +626,7 @@ class InsertTest(_InsertTestBase, fixtures.TablesTest, AssertsCompiledSQL):
             Column("foo", Integer, default=12),
         )
         table1 = self.tables.mytable
-        sel = select([table1.c.myid, table1.c.myid.label("q")]).where(
+        sel = select(table1.c.myid, table1.c.myid.label("q")).where(
             table1.c.name == "foo"
         )
         ins = table.insert().from_select(["id", "foo"], sel)
@@ -554,7 +651,7 @@ class InsertTest(_InsertTestBase, fixtures.TablesTest, AssertsCompiledSQL):
             Column("foo", Integer, default=foo),
         )
         table1 = self.tables.mytable
-        sel = select([table1.c.myid]).where(table1.c.name == "foo")
+        sel = select(table1.c.myid).where(table1.c.name == "foo")
         ins = table.insert().from_select(["id"], sel)
         self.assert_compile(
             ins,
@@ -575,7 +672,7 @@ class InsertTest(_InsertTestBase, fixtures.TablesTest, AssertsCompiledSQL):
             Column("bar", String, default="baz"),
         )
 
-        stmt = select([table_.c.foo])
+        stmt = select(table_.c.foo)
         insert = table_.insert().from_select(["foo"], stmt)
 
         self.assert_compile(stmt, "SELECT mytable.foo FROM mytable")
@@ -593,7 +690,7 @@ class InsertTest(_InsertTestBase, fixtures.TablesTest, AssertsCompiledSQL):
 
     def test_insert_mix_select_values_exception(self):
         table1 = self.tables.mytable
-        sel = select([table1.c.myid, table1.c.name]).where(
+        sel = select(table1.c.myid, table1.c.name).where(
             table1.c.name == "foo"
         )
         ins = self.tables.myothertable.insert().from_select(
@@ -608,7 +705,7 @@ class InsertTest(_InsertTestBase, fixtures.TablesTest, AssertsCompiledSQL):
 
     def test_insert_mix_values_select_exception(self):
         table1 = self.tables.mytable
-        sel = select([table1.c.myid, table1.c.name]).where(
+        sel = select(table1.c.myid, table1.c.name).where(
             table1.c.name == "foo"
         )
         ins = self.tables.myothertable.insert().values(othername="5")
@@ -639,8 +736,8 @@ class InsertTest(_InsertTestBase, fixtures.TablesTest, AssertsCompiledSQL):
 
         name = column("name")
         description = column("desc")
-        sel = select([name, mytable.c.description]).union(
-            select([name, description])
+        sel = select(name, mytable.c.description).union(
+            select(name, description)
         )
         ins = mytable.insert().from_select(
             [mytable.c.name, mytable.c.description], sel
@@ -655,7 +752,7 @@ class InsertTest(_InsertTestBase, fixtures.TablesTest, AssertsCompiledSQL):
     def test_insert_from_select_col_values(self):
         table1 = self.tables.mytable
         table2 = self.tables.myothertable
-        sel = select([table1.c.myid, table1.c.name]).where(
+        sel = select(table1.c.myid, table1.c.name).where(
             table1.c.name == "foo"
         )
         ins = table2.insert().from_select(
@@ -800,7 +897,10 @@ class InsertTest(_InsertTestBase, fixtures.TablesTest, AssertsCompiledSQL):
             "Column 't.id' is marked as a member.*" "may not store NULL.$"
         ):
             self.assert_compile(
-                t.insert(), "INSERT INTO t () VALUES ()", params={}
+                t.insert(),
+                "INSERT INTO t () VALUES ()",
+                params={},
+                supports_default_values=False,
             )
 
 
@@ -811,7 +911,7 @@ class InsertImplicitReturningTest(
 
     def test_insert_select(self):
         table1 = self.tables.mytable
-        sel = select([table1.c.myid, table1.c.name]).where(
+        sel = select(table1.c.myid, table1.c.name).where(
             table1.c.name == "foo"
         )
         ins = self.tables.myothertable.insert().from_select(
@@ -827,7 +927,7 @@ class InsertImplicitReturningTest(
 
     def test_insert_select_return_defaults(self):
         table1 = self.tables.mytable
-        sel = select([table1.c.myid, table1.c.name]).where(
+        sel = select(table1.c.myid, table1.c.name).where(
             table1.c.name == "foo"
         )
         ins = (
@@ -908,17 +1008,44 @@ class EmptyTest(_InsertTestBase, fixtures.TablesTest, AssertsCompiledSQL):
         table1 = self.tables.mytable
 
         stmt = table1.insert().values({})  # hide from 2to3
-        self.assert_compile(stmt, "INSERT INTO mytable () VALUES ()")
+        self.assert_compile(
+            stmt,
+            "INSERT INTO mytable () VALUES ()",
+            supports_default_values=False,
+        )
 
     def test_supports_empty_insert_true(self):
         table1 = self.tables.mytable
 
-        dialect = default.DefaultDialect()
-        dialect.supports_empty_insert = dialect.supports_default_values = True
-
-        stmt = table1.insert().values({})  # hide from 2to3
+        stmt = table1.insert().values({})
         self.assert_compile(
-            stmt, "INSERT INTO mytable DEFAULT VALUES", dialect=dialect
+            stmt,
+            "INSERT INTO mytable DEFAULT VALUES",
+            supports_default_values=True,
+        )
+
+    def test_supports_empty_insert_true_executemany_mode(self):
+        table1 = self.tables.mytable
+
+        dialect = default.DefaultDialect()
+        dialect.supports_empty_insert = False
+        dialect.supports_default_values = True
+        dialect.supports_default_metavalue = True
+
+        stmt = table1.insert().values({})
+        self.assert_compile(
+            stmt,
+            "INSERT INTO mytable (myid) VALUES (DEFAULT)",
+            dialect=dialect,
+            for_executemany=True,
+        )
+
+        dialect.supports_default_metavalue = False
+        self.assert_compile(
+            stmt,
+            "INSERT INTO mytable DEFAULT VALUES",
+            dialect=dialect,
+            for_executemany=True,
         )
 
     def test_supports_empty_insert_false(self):
@@ -936,13 +1063,17 @@ class EmptyTest(_InsertTestBase, fixtures.TablesTest, AssertsCompiledSQL):
             dialect=dialect,
         )
 
-    def _test_insert_with_empty_collection_values(self, collection):
+    @testing.combinations(([],), ({},), ((),))
+    def test_insert_with_empty_collection_values(self, collection):
         table1 = self.tables.mytable
 
         ins = table1.insert().values(collection)
 
         self.assert_compile(
-            ins, "INSERT INTO mytable () VALUES ()", checkparams={}
+            ins,
+            "INSERT INTO mytable () VALUES ()",
+            checkparams={},
+            supports_default_values=False,
         )
 
         # empty dict populates on next values call
@@ -952,15 +1083,6 @@ class EmptyTest(_InsertTestBase, fixtures.TablesTest, AssertsCompiledSQL):
             checkparams={"myid": 3},
         )
 
-    def test_insert_with_empty_list_values(self):
-        self._test_insert_with_empty_collection_values([])
-
-    def test_insert_with_empty_dict_values(self):
-        self._test_insert_with_empty_collection_values({})
-
-    def test_insert_with_empty_tuple_values(self):
-        self._test_insert_with_empty_collection_values(())
-
 
 class MultirowTest(_InsertTestBase, fixtures.TablesTest, AssertsCompiledSQL):
     __dialect__ = "default"
@@ -1012,13 +1134,33 @@ class MultirowTest(_InsertTestBase, fixtures.TablesTest, AssertsCompiledSQL):
             dialect=dialect,
         )
 
-    def test_named_with_column_objects(self):
+    @testing.combinations(("strings",), ("columns",), ("inspectables",))
+    def test_named_with_column_objects(self, column_style):
         table1 = self.tables.mytable
 
+        if column_style == "strings":
+            myid, name, description = "myid", "name", "description"
+
+        elif column_style == "columns":
+            myid, name, description = (
+                table1.c.myid,
+                table1.c.name,
+                table1.c.description,
+            )
+        elif column_style == "inspectables":
+
+            myid, name, description = (
+                ORMExpr(table1.c.myid),
+                ORMExpr(table1.c.name),
+                ORMExpr(table1.c.description),
+            )
+        else:
+            assert False
+
         values = [
-            {table1.c.myid: 1, table1.c.name: "a", table1.c.description: "b"},
-            {table1.c.myid: 2, table1.c.name: "c", table1.c.description: "d"},
-            {table1.c.myid: 3, table1.c.name: "e", table1.c.description: "f"},
+            {myid: 1, name: "a", description: "b"},
+            {myid: 2, name: "c", description: "d"},
+            {myid: 3, name: "e", description: "f"},
         ]
 
         checkparams = {
@@ -1098,6 +1240,62 @@ class MultirowTest(_InsertTestBase, fixtures.TablesTest, AssertsCompiledSQL):
             dialect=dialect,
         )
 
+    def test_mix_single_and_multi_single_first(self):
+        table1 = self.tables.mytable
+
+        stmt = table1.insert().values(myid=1, name="d1")
+        stmt = stmt.values(
+            [{"myid": 2, "name": "d2"}, {"myid": 3, "name": "d3"}]
+        )
+
+        assert_raises_message(
+            exc.InvalidRequestError,
+            "Can't mix single and multiple VALUES formats in one "
+            "INSERT statement",
+            stmt.compile,
+        )
+
+    def test_mix_single_and_multi_multi_first(self):
+        table1 = self.tables.mytable
+
+        stmt = table1.insert().values(
+            [{"myid": 2, "name": "d2"}, {"myid": 3, "name": "d3"}]
+        )
+
+        stmt = stmt.values(myid=1, name="d1")
+
+        assert_raises_message(
+            exc.InvalidRequestError,
+            "Can't mix single and multiple VALUES formats in one "
+            "INSERT statement",
+            stmt.compile,
+        )
+
+    def test_multi_multi(self):
+        table1 = self.tables.mytable
+
+        stmt = table1.insert().values([{"myid": 1, "name": "d1"}])
+
+        stmt = stmt.values(
+            [{"myid": 2, "name": "d2"}, {"myid": 3, "name": "d3"}]
+        )
+
+        self.assert_compile(
+            stmt,
+            "INSERT INTO mytable (myid, name) VALUES (%(myid_m0)s, "
+            "%(name_m0)s), (%(myid_m1)s, %(name_m1)s), (%(myid_m2)s, "
+            "%(name_m2)s)",
+            checkparams={
+                "myid_m0": 1,
+                "name_m0": "d1",
+                "myid_m1": 2,
+                "name_m1": "d2",
+                "myid_m2": 3,
+                "name_m2": "d3",
+            },
+            dialect=postgresql.dialect(),
+        )
+
     def test_inline_default(self):
         metadata = MetaData()
         table = Table(
@@ -1134,7 +1332,8 @@ class MultirowTest(_InsertTestBase, fixtures.TablesTest, AssertsCompiledSQL):
             dialect=postgresql.dialect(),
         )
 
-    def test_python_scalar_default(self):
+    @testing.combinations(("strings",), ("columns",), ("inspectables",))
+    def test_python_scalar_default(self, key_type):
         metadata = MetaData()
         table = Table(
             "sometable",
@@ -1144,10 +1343,23 @@ class MultirowTest(_InsertTestBase, fixtures.TablesTest, AssertsCompiledSQL):
             Column("foo", Integer, default=10),
         )
 
+        if key_type == "strings":
+            id_, data, foo = "id", "data", "foo"
+        elif key_type == "columns":
+            id_, data, foo = table.c.id, table.c.data, table.c.foo
+        elif key_type == "inspectables":
+            id_, data, foo = (
+                ORMExpr(table.c.id),
+                ORMExpr(table.c.data),
+                ORMExpr(table.c.foo),
+            )
+        else:
+            assert False
+
         values = [
-            {"id": 1, "data": "data1"},
-            {"id": 2, "data": "data2", "foo": 15},
-            {"id": 3, "data": "data3"},
+            {id_: 1, data: "data1"},
+            {id_: 2, data: "data2", foo: 15},
+            {id_: 3, data: "data3"},
         ]
 
         checkparams = {
@@ -1196,6 +1408,91 @@ class MultirowTest(_InsertTestBase, fixtures.TablesTest, AssertsCompiledSQL):
             dialect=postgresql.dialect(),
         )
 
+    def test_sql_expression_pk_autoinc_lastinserted(self):
+        # test that postfetch isn't invoked for a SQL expression
+        # in a primary key column.  the DB either needs to support a lastrowid
+        # that can return it, or RETURNING.  [ticket:3133]
+        metadata = MetaData()
+        table = Table(
+            "sometable",
+            metadata,
+            Column("id", Integer, primary_key=True),
+            Column("data", String),
+        )
+
+        stmt = table.insert().return_defaults().values(id=func.foobar())
+        compiled = stmt.compile(dialect=sqlite.dialect(), column_keys=["data"])
+        eq_(compiled.postfetch, [])
+        eq_(compiled.returning, [])
+
+        self.assert_compile(
+            stmt,
+            "INSERT INTO sometable (id, data) VALUES " "(foobar(), ?)",
+            checkparams={"data": "foo"},
+            params={"data": "foo"},
+            dialect=sqlite.dialect(),
+        )
+
+    def test_sql_expression_pk_autoinc_returning(self):
+        # test that return_defaults() works with a primary key where we are
+        # sending a SQL expression, and we need to get the server-calculated
+        # value back.  [ticket:3133]
+        metadata = MetaData()
+        table = Table(
+            "sometable",
+            metadata,
+            Column("id", Integer, primary_key=True),
+            Column("data", String),
+        )
+
+        stmt = table.insert().return_defaults().values(id=func.foobar())
+        returning_dialect = postgresql.dialect()
+        returning_dialect.implicit_returning = True
+        compiled = stmt.compile(
+            dialect=returning_dialect, column_keys=["data"]
+        )
+        eq_(compiled.postfetch, [])
+        eq_(compiled.returning, [table.c.id])
+
+        self.assert_compile(
+            stmt,
+            "INSERT INTO sometable (id, data) VALUES "
+            "(foobar(), %(data)s) RETURNING sometable.id",
+            checkparams={"data": "foo"},
+            params={"data": "foo"},
+            dialect=returning_dialect,
+        )
+
+    def test_sql_expression_pk_noautoinc_returning(self):
+        # test that return_defaults() works with a primary key where we are
+        # sending a SQL expression, and we need to get the server-calculated
+        # value back.  [ticket:3133]
+        metadata = MetaData()
+        table = Table(
+            "sometable",
+            metadata,
+            Column("id", Integer, autoincrement=False, primary_key=True),
+            Column("data", String),
+        )
+
+        stmt = table.insert().return_defaults().values(id=func.foobar())
+        returning_dialect = postgresql.dialect()
+        returning_dialect.implicit_returning = True
+        compiled = stmt.compile(
+            dialect=returning_dialect, column_keys=["data"]
+        )
+        eq_(compiled.postfetch, [])
+        eq_(compiled.returning, [table.c.id])
+
+        self.assert_compile(
+            stmt,
+            "INSERT INTO sometable (id, data) VALUES "
+            "(foobar(), %(data)s) RETURNING sometable.id",
+            checkparams={"data": "foo"},
+            params={"data": "foo"},
+            dialect=returning_dialect,
+        )
+
     def test_python_fn_default(self):
         metadata = MetaData()
         table = Table(
